#ifndef _GAME_H_
#define _GAME_H_

#include "defines.h"
#include "Render.h"
#include "Texture.h"
#include "3dMath.h"
#include "Level.h"
#include "Weapon.h"
#include <vector>

/*	KLASA gameThing
	Jest to klasa wyjciowa dla wikszoci
	obiektw "ywych" -  w tym take dla gracza.
	Posiada odpowiednie metody, by odpowiednio
	zarzdza dan postaci, oraz gotowe algorytmy
	sztucznej inteligencji.
*/
class gameThing : public Dummy
{
protected:
	float Health;
	float MaxHealth;
	float Armor;
	float MaxArmor;
	float Angle;
	float StartAngle;
	float Speed;
	float RotSpeed;
	float DyingTime;
	float ThisDTime;
	bool Dead;
	bool Dying;
	bool HasTarget;
	unsigned int AIflags;
	unsigned int AIState;
	unsigned int Actions;
	unsigned int Type;

	GLModel*	Model;

	Vector3f MoveVector;
	Vector3f StartPos;
	Vector3f Target;

	gameThing* Enemy;

	virtual Vector3f CreatePos( float ang );

	virtual bool AIFindTarget();
	virtual Vector3f AIFindWalkTarget();
	virtual void AIStand();
	virtual void AIWalk();
	virtual void AIAttackTarget();
	virtual bool IsEnemyInFront();
public:
	float ToAngle;
	gameThing();
	~gameThing();

	virtual void Init();

	virtual void DoEngine();
	virtual void DoDraw();

	virtual void OnDie();
	virtual void OnDead();

	virtual void Move( unsigned int flags );
	virtual void DoAI();

	bool IsDead();
	bool IsDying();
	void Reset();

	float GetHealth();
	virtual void ModHealth( float mod );
	virtual void SetHealth( float set );

	float GetArmor();
	virtual void ModArmor( float mod );
	virtual void SetArmor( float set );

	virtual void Fire( Vector3f FireTarget );

	float GetAng();
	void ModAngle( float mod );
	void SetAngle( float set );
	void GoToAngle( float ang );

	void SetStartAngle( float set );
	float GetStartAngle();

	void SetStartPos( Vector3f set );
	Vector3f GetStartPos();
	
	unsigned int GetType();
	virtual void Free();
};


/*	KLASA POSTACI
	Postacie gry maj swoj binarn
	posta w taki sposb. Dziki niej
	mona atwo sterowa i wpywa na ich
	zachowanie.
*/
class gamePlayer : public gameThing
{
private:
	// Posta biega?
	bool run;

	// "Krok" o jaki porusza si posta chodzc
	float WalkStep;

	// ... i biegajc
	float RunStep;
	
	// Aktualna Bro
	unsigned int CurrWeap;
	unsigned int Hand;
public:
	weWeapon* Weapon[10];

	gamePlayer();
	~gamePlayer();

	void DoDraw();
	void DoEngine();
	void DoEngine( bool* Keys );
	void SwichWeap( unsigned int index );

	unsigned int GetHand();

	void Move( unsigned int flags );
	void TestWeapon( weWeapon* Weap );
	void ApplyNextPos();

	void ModHealth( float mod );
};

/*	KLASA PRZECIWNIKW
	Powstaa na bazie gameThing i zadaniem
	tego typu postaci jest walka. Posiada
	odpowiednie wartoci by precyzyjnie umieci
	bro oraz swoje imie i nazwe identyfikacyjn.
*/
class gameEnemy : public gameThing
{
private:
	std::string		file;
	std::string		name;
	std::string		ID;
	unsigned int	WeapType;
	unsigned int	RotObj;
	float	WeapR;
	float	WeapY;
	float	WeapAngle;
	float	FirePause;
	float	FireTime;
	bool loaded;

	std::string GetStr( FILE* fp );
public:
	gameEnemy()
	{	Type = GAME_THING_ENEMY; file = "-";	};

	bool LoadEnemy( std::string filename );

	void DoDraw();
	void Fire( Vector3f FireTarget );

	void OnDie();
	std::string GetID();
};

/*	KLASA gameStatObj
*/
class gameStatObj : public gameThing
{
private:
	std::string		file;

	std::string GetStr( FILE* fp );
public:
	gameStatObj()
	{	file = "-";	};

	void DoEngine();
	void DoDraw();

	bool LoadObj( std::string filename );
};


/*	KLASA gameThingManager
	Jest to klasa zarzdajca wszystkimi
	obiektami na scenie ( poza broni, amunicj i bonusami )
*/
class gameThingManager
{
private:
	std::vector<gameThing*> List;
	unsigned int dead;
	unsigned int life;
	unsigned int all;
public:
	gameThingManager();
	~gameThingManager();

	void DoEngine();
	void DoDraw();

	void ReCountStats();
	void ResetAll();

	void AddThing( gameThing* Thing );
	void DeleteThing( unsigned int index );
	gameThing* GetThing( unsigned int index );
	gameEnemy* GetEnemyByID( std::string ID );
	Vector3f GetThingPos( unsigned int index );
	Vector3f GetThingBlockPos( unsigned int index );
	gameBlockInfo* GetThingBlock( unsigned int index );

	unsigned int Count();
	void Clear();

	unsigned int GetDeadCount();
	unsigned int GetLifeCount();
	unsigned int GetAllCount();
};

/*	KLASA gameWeaponManager
	Suy do zarzdzania broni lec na ziemi
	na poziomie. z tej klasy gwnie korzysta gracz.
	( i nikt poza nim :) )
*/
class gameWeaponManager
{
private:
	std::vector <weWeapon*> List;
public:
	gameWeaponManager();
	~gameWeaponManager();
	void DoEngine( gamePlayer* Players, int PlayerCount );
	void DoDraw();

	void AddWeapon( weWeapon* weapon );
	void DeleteWeapon( unsigned int index );
	weWeapon* GetWeapon( unsigned int index );
	void Clear();
	void LoadFromLevel();
};

/*
class gameStatObj : public Dummy
{
private:
	float rot;
	GLModel* Model;
public:
	gameStatObj();
	~gameStatObj();

	void SetRot( float set );
	float GetRot();

	void LoadFromFile( std::string filename );
	void DoDraw();
};
*/
extern gameThingManager ThingManager;
extern gameWeaponManager WManager;
extern gamePlayer MainPlayer;

#endif