#include "Level.h"

gameLevel GLevel;
gameLevel* pGLevel = &GLevel;

bool gameWLFlags::IsAllDead()
{
	if( ThingManager.GetAllCount() == 0 )
		return false;

	if( ThingManager.GetLifeCount() == 0 )
		return true;
	else return false;

	return false;
}

bool gameWLFlags::IsInBlock()
{
	if( GLevel.GetBlock( MainPlayer.GetBlockPos() ) == Block )
		return true;
	else return false;

	return false;
}

bool gameWLFlags::GotWeap()
{
	return MainPlayer.Weapon[WeapID]->GetHave();	
}

bool gameWLFlags::IsSelfDead()
{
	return MainPlayer.IsDead();
}

bool gameWLFlags::IsThingDead()
{
	if( Enemy == NULL )
		return false;

	return Enemy->IsDead();
}

void gameWLFlags::VerifyFlags()
{
	if( flags & WLFLAG_GO_TO_BLOCK )
	{
		Block = GLevel.GetBlock( BlockID );
	}
	if( flags & WLFLAG_THING_DEAD )
	{
		Enemy = ThingManager.GetEnemyByID( EnemyID );
	}
}

bool gameWLFlags::CheckAllFlags()
{
	bool status = true;
	if( ( flags & WLFLAG_ALL_ENEM_DEAD ) && !IsAllDead() )
		status = false;
	if( ( flags & WLFLAG_GO_TO_BLOCK ) && !IsInBlock() )
		status = false;
	if( ( flags & WLFLAG_GET_WEAP ) && !GotWeap() )
		status = false;
	if( ( flags & WLFLAG_SELF_DEAD ) && !IsSelfDead() )
		status = false;
	if( ( flags & WLFLAG_THING_DEAD ) && !IsThingDead() )
		status = false;
	return status;
}

bool gameWLFlags::CheckOneFlag()
{
	bool status = false;
	if( ( flags & WLFLAG_ALL_ENEM_DEAD ) && IsAllDead() )
		status = true;
	if( ( flags & WLFLAG_GO_TO_BLOCK ) && IsInBlock() )
		status = true;
	if( ( flags & WLFLAG_GET_WEAP ) && GotWeap() )
		status = true;
	if( ( flags & WLFLAG_SELF_DEAD ) && IsSelfDead() )
		status = true;
	if( ( flags & WLFLAG_THING_DEAD ) && IsThingDead() )
		status = true;
	return status;
}


/*	KONSTRUKTOR	*/
gameLevel::gameLevel()
{
	// Zerujemy wskanik (ktry bdzie do tablicy)
	block = NULL;
	// Ustawiamy, e poziom nie by jeszcze adowany
	loaded = false;
	// Ustawiamy domyln nazw poziomu
	LevName = "UNKNOWN LEVEL";
	// Oraz domyln nazw pliku
	file = "-";
	// Ustawiamy, e nie ma adnych kolumn, czy wierszy
	rows = 0;
	cols = 0;
	// I zerujemy list wywietlania
	Top = 0;
	Floor = 0;
	Wall = 0;
	// Zerujemy inne wartoci
	WeapCount = 0;
	AmmoCount = 0;
	StatObjCount = 0;
	BonusCount = 0;
	EnemyCount = 0;
	// Zerujemy liste typw wroga
	EnemyType = NULL;

	Tex[0] = NULL;
	Tex[1] = NULL;
	Tex[2] = NULL;
}

/*	DESTRUKTOR	*/
gameLevel::~gameLevel()
{
	/*	Przy destrukcji wypadao
		by zwolni pami
	*/
	Free();
}

bool gameLevel::GetLoaded()
{
	return loaded;
}

unsigned int gameLevel::GetEnemyCount()
{
	return EnemyCount;
}

unsigned int gameLevel::GetSObjCount()
{
	return StatObjCount;
}

std::string gameLevel::GetString( FILE* fp )
{
	char buf[1024];
	std::string zwrot;
	int len = 0;
	bool repeat = false;
	bool wfchar = false;

	do
	{
		len = 0;
		repeat = false;
		wfchar = false;
		fgets( buf, 1024, fp );
	
		len = strlen( buf );
		zwrot = "";

		if( buf[0] == '\n' )
		{
			repeat = true;
			continue;
		}

		for( int i = 0; i < len; i++ )
		{
			if( ( buf[i] == '#' || buf[i] == '/' ) && !wfchar )
			{
				repeat = true;
				break;
			}

			if( buf[i] != ' ' && buf[i] != '\t' && buf[i] != '\n' )
				wfchar = true;

			zwrot += buf[i];
		}
	}
	while( repeat );

	if( zwrot[zwrot.length()-1] == '\n' )
		zwrot = zwrot.substr( 0, zwrot.length() - 1 );

	return zwrot;
}

std::string gameLevel::GetParamStr( std::string str )
{
	int i;
	bool cancopy = false;
	std::string zwrot;

	for( i = 0; i < str.length(); i++ )
	{
		if( !cancopy && str[i] == '=' )
		{
			cancopy = true;
			continue;
		}

		if( !cancopy )
			continue;

		zwrot += str[i];
	}

	return zwrot;
}

/*	METODA WCZYTUJCA DANE
	Zaoenia pliku s proste.
	Na pocztku podstawowe dane, 
	a dalej jedziemy ze cianami
	i ca reszt danych.
*/
bool gameLevel::LoadLevel( std::string filename )
{
	if( filename == "" )
	{
		Log.Error( "GLEVEL( " + file + " ): acuch znakw jest pusty!" );
		return false;
	}

	// Najpierw definiujemy wskanik na plik i go otwieramy
	FILE* fp = fopen( filename.c_str(), "rt" );

	if( !fp )
	{
		Log.Error( "GLEVEL( " + file + " ): Plik nie istnieje, lub podana cieka jest bdna: " + filename );
		return false;
	}

	// definiujemy zmienne pomocnicze
	std::string str;
	int i, j, k, l;

	// Pobieramy wersje poziomu oraz j sprawdzamy
	str = GetString( fp );
	if( !sscanf( str.c_str(), "E3DTLEV=%u", &Version ) )
	{
		Log.Error( "GLEVEL( " + file + " ): Nieprawidowy plik poziomu!" );
		fclose( fp );
		return false;
	}

	if( Version > GAME_VERSION )
	{
		Log.Error( "GLEVEL( " + file + " ): Zbyt wysoka wersja pliku!" );
		fclose( fp );
		return false;
	}

	/*	Wczenijsze if'y byy, by uzyska pewno,
		e wszystko z nowym plikiem jest w porzdku.
		Dlaczego tyle zachodu? By unikn sytuacji,
		kiedy adujemy plik, na ju istniejcy level.
		Level istnieje, wic go kasujemy i czytamy plik.
		A teraz Zonk! Nie ma pliku! I co? zostajemy na
		lodzie bez levelu :). Teraz wiecie o co chodzi :)
	*/
	if( loaded )
	{
		Log.Report( "GLEVEL( " + file + " ): Przeadowanie poziomu na : " + filename );
		Free();
		ThingManager.Clear();
		WManager.Clear();
		MainPlayer.Reset();
	}
	
	Log.Log( "GLEVEL( " + file + " ): Wczytywanie poziomu: " + filename );
	GUI.SendConMsg( "Wczytywanie poziomu: " + filename, false );
	file = filename;
	
	// Teraz nastpna linia MUSI byc rozpoczciem nagwka, inaczej klops
	str = GetString( fp );
	if( str != "HEADER" )
	{
		Log.Error( "GLEVEL( " + file + " ): Brak nagwka, lub w nieodpowiednim miejscu!" );
		Free();
		fclose( fp );
		return false;
	}
	else
	{
		do
		{
			str = GetString( fp );
			if( guiIsInStr( str, "NAME" ) )
			{
				LevName = GetParamStr( str );
			}
			if( guiIsInStr( str, "ROWS" ) )
			{
				if( !sscanf( str.c_str(), "ROWS=%u", &rows ) )
				{
					Log.Error( "GLEVEL( " + file + " ): Nie mona odczyta liczby wierszy!" );
					Free();
					fclose( fp );
					return false;
				}
			}
			if( guiIsInStr( str, "COLS" ) )
			{
				if( !sscanf( str.c_str(), "COLS=%u", &cols ) )
				{
					Log.Error( "GLEVEL( " + file + " ): Nie mona odczyta liczby kolumn!" );
					Free();
					fclose( fp );
					return false;
				}
			}
			if( guiIsInStr( str, "WEAPCOUNT" ) )
			{
				if( !sscanf( str.c_str(), "WEAPCOUNT=%u", &WeapCount ) )
				{
					Log.Error( "GLEVEL( " + file + " ): Nie mona odczyta liczby broni!" );
					Free();
					fclose( fp );
					return false;
				}
			}
			if( guiIsInStr( str, "ENEMYTYPECOUNT" ) )
			{
				if( !sscanf( str.c_str(), "ENEMYTYPECOUNT=%u", &EnemyTypeCount ) )
				{
					Log.Error( "GLEVEL( " + file + " ): Nie mona odczyta liczby typw wroga!" );
					Free();
					fclose( fp );
					return false;
				}
			}
			if( guiIsInStr( str, "ENEMYCOUNT" ) )
			{
				if( !sscanf( str.c_str(), "ENEMYCOUNT=%u", &EnemyCount ) )
				{
					Log.Error( "GLEVEL( " + file + " ): Nie mona odczyta liczby wrogw!" );
					Free();
					fclose( fp );
					return false;
				}
			}
			if( guiIsInStr( str, "PLAYERSTARTPOS" ) )
			{
				if( !sscanf( str.c_str(), "PLAYERSTARTPOS=%u", &PlayerStartBlock ) )
				{
					Log.Error( "GLEVEL( " + file + " ): Nie mona odczyta pozycji startowej gracza!" );
					Free();
					fclose( fp );
					return false;
				}
			}
			if( guiIsInStr( str, "PLAYERSTARTANGLE" ) )
			{
				if( !sscanf( str.c_str(), "PLAYERSTARTANGLE=%u", &PlayerStartAngle ) )
				{
					Log.Error( "GLEVEL( " + file + " ): Nie mona odczyta obrotu startowego gracza!" );
					Free();
					fclose( fp );
					return false;
				}
			}
			if( guiIsInStr( str, "STATOBJ" ) )
			{
				if( !sscanf( str.c_str(), "STATOBJ=%u", &StatObjCount ) )
				{
					Log.Error( "GLEVEL( " + file + " ): Nie mona odczyta liczby objektw statycznych!" );
					Free();
					fclose( fp );
					return false;
				}
			}
		}
		while( str != "END HEADER" );	
	}
	Log.Log( "GLEVEL( " + file + " ): Nazwa poziomu: " + LevName );

	/*	Dalej pobieramy dane o plikach
		tekstur do poziomw.
		Pierwszy to ciany, drugi sufit,
		trzeci podoga.
	*/
	str = GetString( fp );
	if( str != "TEXTURES" )
	{
		Log.Error( "GLEVEL( " + file + " ): Brak listy tekstur!" );
		Free();
		fclose( fp );
		return false;
	}
	else
	{
		str = GetString( fp );
		if( !( Tex[0] = TManager.Get( GetParamStr( str ) ) ) )
		{
			Log.Error( "GLEVEL( " + file + " ): Bdny plik graficzny!" );
			Free();
			fclose( fp );
			return false;
		}
		str = GetString( fp );
		if( !( Tex[1] = TManager.Get( GetParamStr( str ) ) ) )
		{
			Log.Error( "GLEVEL( " + file + " ): Bdny plik graficzny!" );
			Free();
			fclose( fp );
			return false;
		}
		str = GetString( fp );
		if( !( Tex[2] = TManager.Get( GetParamStr( str ) ) ) )
		{
			Log.Error( "GLEVEL( " + file + " ): Bdny plik graficzny!" );
			Free();
			fclose( fp );
			return false;
		}

		str = GetString( fp );
		if( str != "END TEXTURES" )
		{
			Log.Error( "GLEVEL( " + file + " ): Brak zakoczenia liczby tesktur!" );
			Free();
			fclose( fp );
			return false;
		}
	}

	/*	Teraz tworzymy tablic odpowiednich
		rozmiarw zdoln pomieci nastpne
		dane. S to liczby cakowite jednoznacznie
		okrelajce zawarte w nich ciany ( patrz
		definicje makrowe )
	*/
	str = GetString( fp );
	if( str != "WALLS" )
	{
		Log.Error( "GLEVEL( " + file + " ): Brak listy cian!" );
		Free();
		fclose( fp );
		return false;
	}
	block = new gameBlockInfo[rows*cols];

	for( i = 0; i < rows*cols; i++ )
	{
		str = GetString( fp );
		
		// Wpisujemy t liczb do skadowej walls
		sscanf( str.c_str(), "%d", &block[i].walls );
		block[i].CornerCount = 0;
	}

	str = GetString( fp );
	if( str != "END WALLS" )
	{
		Log.Error( "GLEVEL( " + file + " ): Brak koca listy cian!" );
		Free();
		fclose( fp );
		return false;
	}
	
	if( WeapCount > 0 )
	{
		str = GetString( fp );

		if( str != "WEAPLIST" )
		{
			Log.Error( "GLEVEL( " + file + " ): Brak listy broni!" );
			Free();
			fclose( fp );
			return false;
		}

		for( i = 0; i < WeapCount; i++ )
		{
			str = GetString( fp );

			sscanf( str.c_str(), "%d=%d", &j, &k );

			weWeapon* Weap;
			switch( k )
			{
			case GAME_WEAP_SAW :
				Weap = new weSaw;
				break;

			case GAME_WEAP_PISTOL :
				Weap = new wePistol;
				break;

			case GAME_WEAP_MINIPZR :
				Weap = new weMiniPhazer;
				break;

			case GAME_WEAP_MINIGUN :
				Weap = new weMiniGun;
				break;

			case GAME_WEAP_ROCKETLUN :
				Weap = new weRocketLuncher;
				break;

			case GAME_WEAP_PICKABOO	:
			case GAME_WEAP_PHAZER :
				Weap = new wePhazer;
				break;

			case GAME_WEAP_MINE :
			case GAME_WEAP_ATOM_BOMB :
				break;
			}
			Weap->Init();
			Weap->Pos = GetBlockPos( j );
			Weap->NextPos = Weap->Pos;
			WManager.AddWeapon( Weap );
		}

		str = GetString( fp );
		if( str != "END WEAPLIST" )
		{
			Log.Error( "GLEVEL( " + file + " ): Brak koca listy broni!" );
			Free();
			fclose( fp );
			return false;
		}
	}

	if( EnemyTypeCount > 0 )
	{
		str = GetString( fp );
		EnemyType = new std::string[EnemyTypeCount];

		if( str != "ENEMYTYPELIST" )
		{
			Log.Error( "GLEVEL( " + file + " ): Brak listy typw wroga!" );
			Free();
			fclose( fp );
			return false;
		}

		for( i = 0; i < EnemyTypeCount; i++ )
		{
			str = GetString( fp );
			EnemyType[i] = str;
		}

		str = GetString( fp );
		if( str != "END ENEMYTYPELIST" )
		{
			Log.Error( "GLEVEL( " + file + " ): Brak koca listy typw wroga!" );
			Free();
			fclose( fp );
			return false;
		}
	}

	if( EnemyCount > 0 && EnemyTypeCount > 0)
	{
		str = GetString( fp );

		if( str != "ENEMYLIST" )
		{
			Log.Error( "GLEVEL( " + file + " ): Brak listy wrogw!" );
			Free();
			fclose( fp );
			return false;
		}

		for( i = 0; i < EnemyCount; i++ )
		{
			str = GetString( fp );

			sscanf( str.c_str(), "%d=%d,%d", &j, &k, &l );

			gameEnemy* Enemy = new gameEnemy;
			Enemy->SetStartPos( this->GetBlockPos( j ) );
			Enemy->SetStartAngle( (float)l );
			Enemy->LoadEnemy( EnemyType[k] );
			Enemy->Reset();
			ThingManager.AddThing( Enemy );
		}

		str = GetString( fp );
		if( str != "END ENEMYLIST" )
		{
			Log.Error( "GLEVEL( " + file + " ): Brak koca listy wrogw!" );
			Free();
			fclose( fp );
			return false;
		}
	}

	if( StatObjCount > 0 )
	{
		str = GetString( fp );

		if( str != "STATOBJLIST" )
		{
			Log.Error( "GLEVEL( " + file + " ): Brak listy statycznych objektw!" );
			Free();
			fclose( fp );
			return false;
		}

		for( i = 0; i < StatObjCount; i++ )
		{
			str = GetString( fp );

			sscanf( str.c_str(), "%d,%d=", &k, &l );

			gameStatObj *obj = new gameStatObj;
			obj->LoadObj( GetParamStr( str ) );
			obj->Pos = this->GetBlockPos( k );
			obj->SetAngle( l );
			ThingManager.AddThing( obj );
		}

		str = GetString( fp );
		if( str != "END STATOBJLIST" )
		{
			Log.Error( "GLEVEL( " + file + " ): Brak koca listy statycznych objektw!" );
			Free();
			fclose( fp );
			return false;
		}
	}

	// Zamykamy plik
	str = GetString( fp );
	if( str != "END E3DTLEV" )
	{
		Log.Error( "GLEVEL( " + file + " ): Brak koca pliku!" );
	}
	fclose( fp );

	MainPlayer.SetStartPos( this->GetBlockPos( PlayerStartBlock ) );
	MainPlayer.SetStartAngle( (float)PlayerStartAngle );
	MainPlayer.Reset();

	loaded = true;

	return true;
}

/*	METODA RYSUJCA - CIANY
	W zalenoci od tego jak
	ciane chcemy, tak rysuje
	razem z koordynatami tekstur
*/
void gameLevel::DrawWall( unsigned int wall )
{
	float mult = GUI.GetTexPerPlane( GAME_TEX_WALL );
	switch( wall )
	{
	case LEV_WALL_TOP :
		{
			glBegin( GL_TRIANGLE_STRIP );
			glNormal3f( 0.0f, 0.0f, 1.0f );
			glTexCoord2f( mult, mult );
			glVertex3f( -5.0f, 5.0f, -5.0f );
			glTexCoord2f( mult, 0.0f );
			glVertex3f( -5.0f, -5.0f, -5.0f );
			glTexCoord2f( 0.0f, mult );
			glVertex3f( 5.0f, 5.0f, -5.0f );
			glTexCoord2f( 0.0f, 0.0f );
			glVertex3f( 5.0f, -5.0f, -5.0f );
			glEnd();
		}
		break;
	case LEV_WALL_BOTTOM :
		{
			glBegin( GL_TRIANGLE_STRIP );
			glNormal3f( 0.0f, 0.0f, -1.0f );
			glTexCoord2f( mult, mult );
			glVertex3f( 5.0f, 5.0f, 5.0f );
			glTexCoord2f( mult, 0.0f );
			glVertex3f( 5.0f, -5.0f, 5.0f );
			glTexCoord2f( 0.0f, mult );
			glVertex3f( -5.0f, 5.0f, 5.0f );
			glTexCoord2f( 0.0f, 0.0f );
			glVertex3f( -5.0f, -5.0f, 5.0f );
			glEnd();
		}
		break;
	case LEV_WALL_LEFT :
		{
			glBegin( GL_TRIANGLE_STRIP );
			glNormal3f( 1.0f, 0.0f,0.0f );
			glTexCoord2f( mult, mult );
			glVertex3f( -5.0f, 5.0f, 5.0f );
			glTexCoord2f( mult, 0.0f );
			glVertex3f( -5.0f, -5.0f, 5.0f );
			glTexCoord2f( 0.0f, mult );
			glVertex3f( -5.0f, 5.0f, -5.0f );
			glTexCoord2f( 0.0f, 0.0f );
			glVertex3f( -5.0f, -5.0f, -5.0f );
			glEnd();
		}
		break;
	case LEV_WALL_RIGHT :
		{
			glBegin( GL_TRIANGLE_STRIP );
			glNormal3f( -1.0f, 0.0f, 0.0f );
			glTexCoord2f( mult, mult );
			glVertex3f( 5.0f, 5.0f, -5.0f );
			glTexCoord2f( mult, 0.0f );
			glVertex3f( 5.0f, -5.0f, -5.0f );
			glTexCoord2f( 0.0f, mult );
			glVertex3f( 5.0f, 5.0f, 5.0f );
			glTexCoord2f( 0.0f, 0.0f );
			glVertex3f( 5.0f, -5.0f, 5.0f );
			glEnd();
		}
		break;
	}
}

/*	METODA RYSUJCA - PODOGA I SUFIT
	Metody podobne do poprzedniej - rysuj
	podoge i sufit razem z koordynatami.
*/
void gameLevel::DrawFloor()
{
	float mult = GUI.GetTexPerPlane( GAME_TEX_FLOOR );
	glBegin( GL_TRIANGLE_STRIP );
	glNormal3f( 0.0f, 1.0f, 0.0f );
	glTexCoord2f( mult, mult );
	glVertex3f( -5.0f, -5.0f, -5.0f );
	glTexCoord2f( mult, 0.0f );
	glVertex3f( -5.0f, -5.0f, 5.0f );
	glTexCoord2f( 0.0f, mult );
	glVertex3f( 5.0f, -5.0f, -5.0f );
	glTexCoord2f( 0.0f, 0.0f );
	glVertex3f( 5.0f, -5.0f, 5.0f );
	glEnd();
}

void gameLevel::DrawTop()
{
	float mult = GUI.GetTexPerPlane( GAME_TEX_TOP );
	glBegin( GL_TRIANGLE_STRIP );
	glNormal3f( 0.0f, -1.0f, 0.0f );
	glTexCoord2f( mult, mult );
	glVertex3f( 5.0f, 5.0f, -5.0f );
	glTexCoord2f( mult, 0.0f );
	glVertex3f( 5.0f, 5.0f, 5.0f );
	glTexCoord2f( 0.0f, mult );
	glVertex3f( -5.0f, 5.0f, -5.0f );
	glTexCoord2f( 0.0f, 0.0f );
	glVertex3f( -5.0f, 5.0f, 5.0f );
	glEnd();
}

/*	Ta metoda jednorazowo wywouje dwie
	inne metody
*/
void gameLevel::InitLevel()
{
	// Gdy poziom ju jest zaadowany, to trzeba przerwa inicjalizacje
	if( !loaded )
		return;

	BuildVisual();
	BuildPhisic();
}

/*	W tej metodzie budujemy cz wizualn
	poziomu, oddzielnie od reszty logiki. W programowaniu gier jest ZAWSZE
	zasada odzielania tych czci.
*/
void gameLevel::BuildVisual()
{
	if( !loaded )
		return;

	GUI.SendConMsg( "Poziom: Tworzenie czesci wizualnej...", false );
	// Zmienne pomocnicze
	int i, j, blk;

	// Generujemy list
	Top = glGenLists( 1 );
	Wall = glGenLists( 1 );
	Floor = glGenLists( 1 );
	/*	Teraz tworzymy list. Listy wywietlania
		maj te zalete, e raz stworzone BARDZO
		przyspieszaj wywietlanie, dlatego opaca
		si je stosowa. W zamian nie mona nic zmieni
		w licie, no chyba, e tworzc j od nowa.
	*/
	/*	Ta cz kodu tysuje podoge, na caej
		powieszchni poziomu.
	*/
	glNewList( Floor, GL_COMPILE );
	glPushMatrix();
	glTranslatef( 5, 0, -5 );
	for( i = 0; i < rows; i++ )
	{
		for( j = 0; j < cols; j++ )
		{
			glPushMatrix();

			glTranslatef( j*10, 0, -(i*10) );

			DrawFloor();

			glPopMatrix();
		}
	}
	glPopMatrix();
	glEndList();
	// To samo co poprzednio, tylko, e z sufitem
	glNewList( Top, GL_COMPILE );
	glPushMatrix();
	glTranslatef( 5, 0, -5 );
	for( i = 0; i < rows; i++ )
	{
		for( j = 0; j < cols; j++ )
		{
			glPushMatrix();

			glTranslatef( j*10, 0, -(i*10) );

			DrawTop();

			glPopMatrix();
		}
	}
	glPopMatrix();
	glEndList();
	/*	W tej ptli s rysowane ciany
		w zalenoci jaka ciana jest zawarta
		w fladze, tak rysujemy.
	*/
	glNewList( Wall, GL_COMPILE );
	glPushMatrix();
	glTranslatef( 5, 0, -5 );
	for( i = 0; i < rows; i++ )
	{
		for( j = 0; j < cols; j++ )
		{
			glPushMatrix();

			glTranslatef( j*10, 0.0f, -(i*10) );

			int in = i * cols + j;

			blk = block[ in ].walls;

			if( blk & LEV_WALL_TOP )
				DrawWall( LEV_WALL_TOP );

			if( blk & LEV_WALL_BOTTOM )
				DrawWall( LEV_WALL_BOTTOM );

			if( blk & LEV_WALL_LEFT )
				DrawWall( LEV_WALL_LEFT );

			if( blk & LEV_WALL_RIGHT )
				DrawWall( LEV_WALL_RIGHT );

			glPopMatrix();
		}
	}
	glPopMatrix();
	glEndList();
}

/*	Ta metoda jest odpowiedzilna, za cz fizyczn poziomu
	buduje, m.in. punkty i paszczyzny, w oparciu o ktre
	bd wykrywane kolizje.
*/
void gameLevel::BuildPhisic()
{
	if( !loaded )
		return;

	GUI.SendConMsg( "Poziom: Tworzenie czesci fizycznej...", false );
	// Zmienne pomocznicze
	int i, j, k, ai, aj;
	gameBlockInfo* blk;
	gameBlockInfo* SideBlock[4];
	float nx, ny;

	/*	Ptla sprawdza kade 4 ciany
		w kadym bloku, czy naley stworzy
		w tym miejscu przeszkode.
	*/
	for( i = 0; i < rows; i++ )
	{
		for( j = 0; j < cols; j++ )
		{
			blk = &block[ i * cols + j ];
			
			for( k = 0; k < 4; k++ )
			{
				/*	BTW: tu mona byo by wymyle jaki algorytm,
					ale nie miaem pomysu, wic po prostu posuyem
					si switch'em.
				*/
				switch( k )
				{
					/*	Wybieramy jakie ustawienia wybra dla danej ciany.
						nx, ny - wektor normalny ciany ( wskazuje gdzie jest przd )
						ai, aj - dodatek do punktu dolnego, prawego ( bo jako trzeba ustawi rogi
						bloku )
					*/
				case 0 :
					nx = 0.0f;
					ny = 1.0f;
					ai = 0;
					aj = 1;
					break;
				case 1 :
					nx = -1.0f;
					ny = 0.0f;
					ai = 1;
					aj = 1;
					break;
				case 2 :
					nx = 0.0f;
					ny = -1.0f;
					ai = 1;
					aj = 0;
					break;
				case 3 :
					nx = 1.0f;
					ny = 0.0f;
					ai = 0;
					aj = 0;
					break;
				}
				
				// Najpierw tworzymy odpowiednie rogi bloku.
				blk->Corner[k].Set( (float)j * 10.0f + (float)( ai * 10 ), 0.0f, -(float)i * 10.0f + (float)( aj * -10 ) );
				// Tworzymy wektor normalny paszczyzny ciany
				blk->WPlane[k].Normal.Set( nx, 0.0f, ny );
				// Wyliczamy przesunicie paszczyzny, w oparciu o Punkt ( w tym przypadku rg bloku ) i Wektor Normalny
				blk->WPlane[k].D = -blk->Corner[k].Dot( blk->WPlane[k].Normal );
			}
		}
	}

	/*	Zauwayem, e kiedy teraz prbowabym 
		wej na jaki rg na cianach, to bym 
		po prostu... przeszed na druga stron!
		Co takiego nie moe mie miejsca, dlatego
		tworzymy odpowiednie rogi "kolizyjne". 
		A dlaczego nie wykorzystamy ju wczeniej
		stworzonych?? Bo sytuacja gdy si blokujemy
		na rodku pustej przestrzenie, te jest 
		absurdalna :)
	*/
	for( i = 0; i < rows; i++ )
	{
		for( j = 0; j < cols; j++ )
		{
			blk = &block[ i * cols + j ];

			/*	Znajdujemy bloki ssiednie, by mona
				byo znale rogi cian.
			*/
			if( i < rows - 1 )
				SideBlock[0] = &block[ (i+1) * cols + j ];
			else
				SideBlock[0] = NULL;

			if( j < cols - 1 )
				SideBlock[1] = &block[ i * cols + (j+1) ];
			else
				SideBlock[1] = NULL;

			if( i > 0 )
				SideBlock[2] = &block[ (i-1) * cols + j ];
			else
				SideBlock[2] = NULL;

			if( j > 0 )
				SideBlock[3] = &block[ i * cols + (j-1) ];
			else
				SideBlock[3] = NULL;

			blk->CornerCount = 0;
			/*	Teraz na kad ciany ( i jednoczenie blok ssiedni ze cian )
				szukamy zetknicia lub rogu. Gdy taki znajdujemy, to zapisujemy
				ten rg i zwikszamy liczb rogw do sprawdzenia.
			*/
			for( k = 0; k < 4; k++ )
			{
				if( k == 0 )
					if( ( SideBlock[0] == NULL || !( SideBlock[0]->walls & LEV_WALL_RIGHT ) ) && 
						( SideBlock[1] == NULL || !( SideBlock[1]->walls & LEV_WALL_TOP ) ) )
						continue;

				if( k == 1 )
					if( ( SideBlock[1] == NULL || !( SideBlock[1]->walls & LEV_WALL_BOTTOM ) ) && 
						( SideBlock[2] == NULL || !( SideBlock[2]->walls & LEV_WALL_RIGHT ) ) )
						continue;

				if( k == 2 )
					if( ( SideBlock[2] == NULL || !( SideBlock[2]->walls & LEV_WALL_LEFT ) ) && 
						( SideBlock[3] == NULL || !( SideBlock[3]->walls & LEV_WALL_BOTTOM ) ) )
						continue;

				if( k == 3 )
					if( ( SideBlock[3] == NULL || !( SideBlock[3]->walls & LEV_WALL_TOP ) ) && 
						( SideBlock[0] == NULL || !( SideBlock[0]->walls & LEV_WALL_LEFT ) ) )
						continue;

				if( SideBlock[k] != NULL )
					blk->TCorner[blk->CornerCount] = SideBlock[k]->Corner[(k+2)%4];
				else blk->TCorner[blk->CornerCount] = SideBlock[(k+1)%4]->Corner[k];

				blk->CornerCount++;
				
			}

		}
	}

}

/*	Tu odbywa si rysowanie levelu. 
	( takie proste, e a gupie :) )
*/
void gameLevel::DrawLevel()
{
	if( !loaded )
		return;

	if( GUI.GetReflection() )
	{
		this->DrawAllTop();
		this->DrawAllWall();

		glPushMatrix();
		glTranslatef( 0.0f, -10.0f, 0.0f );
		glScalef( 1.0f, -1.0f, 1.0f );
		glDisable( GL_CLIP_PLANE0 );
		glEnable( GL_CLIP_PLANE1 );
		glFrontFace( GL_CW );
		this->DrawAllTop();
		this->DrawAllWall();
		glFrontFace( GL_CCW );
		glDisable( GL_CLIP_PLANE1 );
		glEnable( GL_CLIP_PLANE0 );
		glPopMatrix();
	}
	else
	{
		this->DrawAllTop();
		this->DrawAllWall();
		this->DrawAllFloor();
	}
}

void gameLevel::DrawAllTop()
{
	if( !loaded )
		return;

	Tex[1]->Activate( GUI.GetTexDLevel() );
	glCallList( Top );
}

void gameLevel::DrawAllWall()
{
	if( !loaded )
		return;

	Tex[0]->Activate( GUI.GetTexDLevel() );
	glCallList( Wall );
}

void gameLevel::DrawAllFloor()
{
	if( !loaded )
		return;

	Tex[2]->Activate( GUI.GetTexDLevel() );
	glCallList( Floor );
}

void gameLevel::DrawReflect()
{
	if( !GUI.GetReflection() )
		return;

	glPushMatrix();
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
	glColor4f( 1.0f, 1.0f, 1.0f, GUI.GetRefLevel() );
	glEnable( GL_BLEND );
	this->DrawAllFloor();
	glColor4f( 1.0f, 1.0f, 1.0f, 1.0f );
	glDisable( GL_BLEND );
	glPopMatrix();
}

/*	Ta metoda zwraca nazw poziomu wyczytan z pliku
	(lub domyln).
*/
std::string gameLevel::GetLevelName()
{
	return LevName;
}

/*	Ta zwraca dany block z okrelonego wiersza i kolumny
*/
gameBlockInfo* gameLevel::GetBlock( unsigned int i, unsigned int j )
{
	if( i >= 0 && j >= 0 )
	{
		if( j < rows && i < cols )
		{
			return &block[ j * cols + i ];
		}
	}
	return NULL;
}

gameBlockInfo* gameLevel::GetBlock( Vector3f Pos )
{
	return GetBlock( (unsigned int)Pos.X, (unsigned int)Pos.Z );
}

gameBlockInfo* gameLevel::GetBlock( unsigned int i )
{
	return &block[i];
}

unsigned int gameLevel::GetBlockCount()
{
	return rows*cols;
}

Vector3f gameLevel::GetBlockPos( unsigned int i )
{
	return Vector3f( (float)(i % cols)*10.0f+5.0f, 0.0f, -(float)(i / cols)*10.0f-5.0f );
}

/*	Metoda czyci pamic zarezerwowan dla
	tej klasy.
*/
void gameLevel::Free()
{
	if( !loaded )
		return;

	GUI.SendConMsg( "Poziom: Zwalnianie pamieci...", false );

	if( block != NULL )
	{
		delete[] block;
		block = NULL;
	}

	if( EnemyType != NULL )
	{
		delete[] EnemyType;
		EnemyType = NULL;
	}

	rows = 0;
	cols = 0;
	LevName = "UNKNOWN LEVEL";

	glDeleteLists( Top, 1 );
	Top = 0;
	glDeleteLists( Wall, 1 );
	Wall = 0;
	glDeleteLists( Floor, 1 );
	Floor = 0;

	Tex[0] = NULL;
	Tex[1] = NULL;
	Tex[2] = NULL;

	loaded = false;
}

/*	To jest odzielna funkcja sprawdzajca kolizje
	obiektu Dummy, ze cianami podanego bloku.
	Jest to samodzielna funkcja, by mona byo j wygonie
	uy, nie korzystajc z klasy gameLevel.
*/
bool TestCollBlock( Dummy* Dum, gameBlockInfo* Block )
{
	if( Block == NULL )
		return false;

	bool is = false;

	// zmienne pomocnicze
	int i;
	Vector3f A, B, Normal, V, P;
	float wynik, D, W;

	/*	A teraz troche trygonometrii :)
		Aby wykry kolizje, w grach 3d uywa
		sie sprawdzenia przecicia paszczyzny przez
		sfer ( kul ). Polega ona, e do poorzenia kuli
		dodajemy wektor o dugoci rwnej promieniowy kuli.
		Cae sprawdzenie opiera si na dwch takich punktach
		aktualnej pozycji i nastpnej pozycji. Nastepnie spraw-
		dzamy poorzenie tych dwch punktw wzgldem paszczyzny
		ciany. Jeeli le po tej samej stronie, to nie ma kolizji,
		a jeeli po rnych, to kolizja nastpi(a). W grach 3D dodatkowo
		sprawdza si paszczyzny na krawdziach ciany, by stwierdzi czy 
		kolizja nastpia na cianie ( bo sama paszczyzna nie jest
		ograniczona i moe si rociga poza cian ), ale tu
		operujemy na 4 cianach, w dodatku pewnie ograniczonych, wic
		nie zachodzi taka potrzeba :)
	*/
	for( i = 0; i < 4; i++ )
	{
		if( i == 0 && !( Block->walls & LEV_WALL_TOP ) )
			continue;
		if( i == 1 && !( Block->walls & LEV_WALL_RIGHT ) )
			continue;
		if( i == 2 && !( Block->walls & LEV_WALL_BOTTOM ) )
			continue;
		if( i == 3 && !( Block->walls & LEV_WALL_LEFT ) )
			continue;

		Normal = Block->WPlane[i].Normal;
		D = Block->WPlane[i].D;

		/* FIXME: nie wiem dlaczego, ale jeeli aktualna
			pozycja ma dodany promie to czsto kolizja
			nie jest wykryta (?) kiedy to musze naprawi...
		*/
		// Aktualna pozycja
		A = Dum->Pos;// + Normal.Reverse() * R;
		// Nastpna pozycja
		B = Dum->NextPos + Normal.Reverse() * Dum->R;

		// Wyraenie liczy, gdzie s punkty wzgldem paszczyzny
		wynik = ( Normal.Dot( A ) + D ) * ( Normal.Dot( B ) + D );

		// wynik > 0 - po tej samej stronie
		// wynik < 0 - po rnych stronach
		if( wynik >= 0 ) 
			continue;

		/*	No dobra, wiemy, e nastpi(a) kolizja. Co dalej??
			Najlepiej wyliczy punkt przecicia paszczyzny i 
			wyliczy now pozycj nastpnego ruchu w oparciu
			o wektor normalny ciany o dugoci promienia.
			To dziaa i nie wyglda sztucznie :)
		*/
		// Wyliczamy punkt przecicia
		V = A - B;
		W = ( Normal.Dot( B ) + D ) / Normal.Dot( V );
		P = B - V * W;

		is = true;
		// i tworzymy now pozycj.
		Dum->NextPos = P + Normal * Dum->R;
	}

	/*	No dobra, wykrywamy kolizje na cianach, ale
		co zrobi gdy block nie ma cian, a w ssiednich
		jest rg? Najlepiej sprawdzi czy odlego
		rogu od nastpnej pozycji nie jest mniejsza od
		promienia :). Wtedy wsytarczy policzy now pozycje
		w dokadnej odlegoci promienia od rogu i wszystko
		jest adnie zrobione :)
	*/
	
	Planef Plane;
	for( i = 0; i < Block->CornerCount; i++ )
	{
		if( fabsf( mathDist( Dum->NextPos, Block->TCorner[i] ) ) > Dum->R )
			continue;

		
		// liczymy wektor od rogu, do pozycji
		V = Dum->NextPos - Block->TCorner[i];
		// i go normalizujemy, by si nadawa do wyliczenia odlegoci
		V.Normalize();
		Plane.Normal = V;
		Plane.D = -Block->TCorner[i].Dot( V );
		Normal = Plane.Normal;
		D = Plane.D;

		// Aktualna pozycja
		A = Dum->Pos;// + Normal.Reverse() * R;
		// Nastpna pozycja
		B = Dum->NextPos + Normal.Reverse() * Dum->R;

		// Powtarzamy poprzedni reguke...
		wynik = ( Normal.Dot( A ) + D ) * ( Normal.Dot( B ) + D );
		if( wynik >= 0 ) 
			continue;
		V = A - B;
		W = ( Normal.Dot( B ) + D ) / Normal.Dot( V );
		P = B - V * W;

		is = true;
		// i tworzymy now pozycj.
		Dum->NextPos = P + Normal * Dum->R;
/*
		is = true;
		// Liczymy now odlego
		Dum->NextPos = Block->TCorner[i] + V * Dum->R;*/
	}
	

	return is;
}

/*	Ta funkcja wykorzystuje technik zwan Ray Casting
	czyli wypuszczanie/puszczanie promieni. Polega ona
	na tym, by z punktu pocztkowego w danym kierunku,
	co okrelony krok bada kolizje, a do znalezienia
	punktu kolizji. Kolizje s sprawdzane za pomoc
	poprzedniej funkcji.
*/
Vector3f RayCast( Vector3f Pos, Vector3f Veloc, float Step, gameLevel* Level )
{
	Dummy Dum;
	Dum.R = 0.1f;
	Dum.NextPos = Pos;
	gameBlockInfo* Block = NULL;

	do
	{
		Dum.Pos = Dum.NextPos;
		Dum.NextPos = Dum.Pos + Veloc * Step;

		Block = Level->GetBlock( Dum.GetBlockPos() );
		/*	Takie mae zabezpieczenie.
			Kiedy promie znajdzie si poza poziomem
			( gdzie nie ma adnych cian ) to zamiast
			si zawiesi, to niech zwrci bezpieczny
			punkt rodka (0, 0, 0).
		*/
		if( Block == NULL )
		{
			return Vector3f( 0.0f, 0.0f, 0.0f );
		}
	}
	while( !TestCollBlock( &Dum, Block ) );

	return Dum.NextPos;
}

/*	Ta funckja jest tak atwa, e a mieszna.
	sprawdza ona odlego midzy dwoma kukami i
	zwraca czy si zdeyy, czy nie.
*/
bool TestCollDum( Dummy* Dum, Dummy* Dum2 )
{
	Vector3f V1 = ClosestPoint( Dum2->Pos, Dum2->NextPos, Dum->NextPos );

	//GUI.SendMsg( "V: " + guiFloatToStr( V1.X ) + " " + guiFloatToStr( V1.Y ) + " " + guiFloatToStr( V1.Z ) );

	if( mathDist( V1, Dum->NextPos ) < Dum->R )
		return true;
	else return false;
}

bool IsCollOnRay( Vector3f V1, Vector3f V2, int Steps )
{
	Vector3f Veloc = V2 - V1;
	Vector3f Step = Veloc / (float)Steps;
	Dummy Dum;
	Dum.R = 0.1f;
	Dum.NextPos = V1;
	gameBlockInfo* Block = NULL;

	for( int i = 0; i < Steps; i++ )
	{
		Dum.Pos = Dum.NextPos;
		Dum.NextPos = Dum.Pos + Step;
		Block = GLevel.GetBlock( Dum.GetBlockPos() );
		
		if( Block == NULL )
			return true;

		if( TestCollBlock( &Dum, Block ) )
			return true;
	}

	return false;
}