#ifndef _LEVEL_H_
#define _LEVEL_H_

#include "defines.h"
#include "Log.h"
#include "gui.h"
#include "Render.h"	// patrz-> nagwek Render.h i plik Render.cpp
#include "Texture.h" // patrz-> nagwek Texture.h i plik Texture.cpp
#include "3dMath.h"	// patrz-> nagwek 3dMath.h i plik 3dMath.cpp
#include "Game.h"
#include <string>	// Nagwek od klasy acucha znakw
#include <stdio.h>	// Nagwek od operacji we/wy.

/*	DEFINICJE MAKROWE
	Te synonimy s dla uatwienia.
	Dziki nim poprawia si czytelno
	kodu.
*/
#define	LEV_WALL_TOP	1
#define	LEV_WALL_BOTTOM	2
#define	LEV_WALL_LEFT	4
#define	LEV_WALL_RIGHT	8

#define	WLFLAG_ALL_ENEM_DEAD	1
#define WLFLAG_GO_TO_BLOCK		2
#define	WLFLAG_GET_WEAP			4
#define WLFLAG_SELF_DEAD		8
#define WLFLAG_THING_DEAD		16

class gameWLFlags
{
private:
	bool IsAllDead();
	bool IsInBlock();
	bool GotWeap();
	bool IsSelfDead();
	bool IsThingDead();
public:
	unsigned int	flags;
	unsigned int	BlockID;
	gameBlockInfo*	Block;
	std::string		EnemyID;
	gameEnemy*		Enemy;
	unsigned int	WeapID;
	std::string		Text;
	float			Color[3];

	void VerifyFlags();
	bool CheckAllFlags();
	bool CheckOneFlag();
};

/*	KLASA gameBlockInfo
	Przechowuje dane potrzebne
	do stworzenia, inicjalizacji
	i dziaani poszczeglnych
	"blokw" na poziomie.
*/
class gameBlockInfo
{
public:
	// Wektor trzymajcy rogi sektora
	Vector3f Corner[4];
	// Wektor trzyma rogi, o ktre gracz ma si zatrzymywa (rogi cian)
	Vector3f TCorner[4];
	// Paszczyzna kadej ciany (do wykrywania kolizji)
	Planef WPlane[4];
	// Flagi cian jakie posiada dany blok
	unsigned int walls;
	// Liczba rogw o ktre moe zawadzi gracz ( od 0 do 4, wie si z TCorner )
	unsigned int CornerCount;

	// Dane dla broni i amunicji ( -1 jak nie ma adnej w danym bloku )
	std::string StatObj;
};

/*	KLASA gameLevel
	Obiekty tej klasy aduj
	pliki txt z poziomami, i
	tworz ich wizualn wersj.
*/
class gameLevel
{
private:
	ioTexture *Tex[3];
	
	// Informacje o liczbie wierszy i kolumn w poziomie gry
	unsigned int rows;
	unsigned int cols;

	unsigned int PlayerStartBlock;
	unsigned int PlayerStartAngle;

	unsigned int WeapCount;
	unsigned int AmmoCount;
	unsigned int BonusCount;
	unsigned int StatObjCount;
	unsigned int EnemyCount;
	unsigned int EnemyTypeCount;
	std::string* EnemyType;

	unsigned int Version;

	// Tablica trzymajca dane wycignite z pliku
	gameBlockInfo* block;

	// Listy wywietlania
	unsigned int Floor;
	unsigned int Top;
	unsigned int Wall;
	unsigned int Reflect;
	// Czy poziom ju jest zaadowany
	bool loaded;

	gameWLFlags WinFlags;
	gameWLFlags LoseFlags;

	// Nazwa poziomu
	std::string LevName;
	// Plik ostatnio zaadowanego poziomu
	std::string file;

	// Metoda pobiera jedna linie z pliku
	std::string GetString( FILE* fp );
	std::string GetParamStr( std::string str );

	// Metody wykorzystywane przy wizualizacji
	void DrawWall( unsigned int wall );
	void DrawFloor();
	void DrawTop();

	// Metody inicjujce
	void BuildVisual();
	void BuildPhisic();
public:
	// Konstruktor i destruktor
	gameLevel();
	~gameLevel();

	std::string GetLevelName();

	unsigned int GetBlockCount();
	gameBlockInfo* GetBlock( unsigned int i, unsigned int j );
	gameBlockInfo* GetBlock( Vector3f Pos );
	gameBlockInfo* GetBlock( unsigned int i );
	Vector3f GetBlockPos( unsigned int i );
	bool GetLoaded();
	unsigned int GetEnemyCount();
	unsigned int GetSObjCount();

	// Metoda adujaca poziom z pliku txt
	bool LoadLevel( std::string filename );

	// Kocowa metoda inicjujca
	void InitLevel();

	// Metoda rysujca
	void DrawLevel();
	void DrawAllWall();
	void DrawAllFloor();
	void DrawAllTop();

	void DrawReflect();

	// Metoda czyszczca pami
	void Free();
};

extern bool TestCollBlock( Dummy* Dum, gameBlockInfo* Block );
extern Vector3f RayCast( Vector3f Pos, Vector3f Veloc, float Step, gameLevel* Level );
extern bool TestCollDum( Dummy* Dum, Dummy* Dum2 );
extern bool IsCollOnRay( Vector3f V1, Vector3f V2, int Steps = 100 );

extern gameLevel GLevel;
extern gameLevel* pGLevel;

#endif