/*--------------------------------------------
  Nazwa:	Nagwek Render.h
  Autor:	Real_Noname
  WWW:      www.coderulers.prv.pl
  Data:		07-05-2005 21:21
  Opis:		Funkcje i klasy potrzebne do sterowania
			OpenGL.
--------------------------------------------*/

#ifndef _RENDER_H_
#define _RENDER_H_

#include <windows.h>	// Nagwek funkcji Windows
#include <gl/gl.h>		// Nagwek funkcji OpenGL
#include <gl/glu.h>		// Nagwek funkcji pomocniczych do OpenGL
//#include <gl/wglext.h>
#include <string>		// Nagowek do mainpulacji acuchem znakw
#include "Log.h"

#define RENDER_VERSION	0
#define RENDER_RENDERER	1
#define RENDER_VENDOR	2
#define RENDER_EXTLIST	3

class UIRender
{
private:
	HGLRC	hRC;	// Kontekst Renderujcy
	HDC		hDC;	// Kontekst Urzdzenia
	HWND*	hWnd;	// Wskaznik na uchwyt okna	
	HINSTANCE	hInstance;	// Instancja

	bool IsPers; // Do sprawdzania, czy jest ustawiona perspektywa
	bool fullS; // Do sprawdzania, czy jest wczony peny Ekran
	bool IsWin;	// Do sprawdzenia, czy okno ju jest postawione
	int	 ScrSize[2]; // Trzyma rozmiar okna. ( 0 - szeroko, 1 - wysoko )
	int	 ColBits; // Bity kolorw

	float PerConf[4]; // Trzyma konfiguracje perspektywy
	/*	PerConf[0] - Kat
		PerConf[1] - Wspczynik pomidzy wysokoscia/szerokoscia
		PerConf[2] - Minimalna odleglosc
		PerConf[3] - Maksymalna odleglosc*/

	float OrthoConf[6]; // Trzyma konfiguracje rzutowania prostoktnego
	/*	OrthoConf[0] - krawedz lewa bryly obcinania
		OrthoConf[1] - prawa
		OrthoConf[2] - dolna
		OrthoConf[3] - grna
		OrthoConf[4] - bliska ( do ciebie od ekranu )
		OrthoConf[5] - daleka ( za ekranem )
	*/

	std::string RndInfo[4];	// Trzyma informacje o danej Maszynie
	/*	RndInfo[0] - Wersja OpenGL
		RndInfo[1] - Sterownik Karty Grafiki
		RndInfo[2] - Firma, ktra wmontowaa OpenGL do karty ( np. NVDIA, ATI )
		RndInfo[3] - Lista rozszerze dostpnych na danej karcie
	*/

public:
	UIRender();		// Konstruktor Klasy
	~UIRender();	// Destruktor Klasy

	// Skalowanie ViewPortu ( tego kwadratu, wktrym wywietlana jest grafika ).
	void Resize( unsigned int szerokosc, unsigned int wysokosc );

	// Ustawianie uchwytu do okna ( WNE: Naley wykona PRZED stworzeniem okna )
	void SetWinHandle( HWND* Uchwyt );

	// Tworzenie Okna ( zwraca true, jezeli udane lub false, jezeli sie nie powiodo )
	bool GLCreateWindow( std::string Tytul, int Szerokosc, int Wysokosc, bool FullScr, WNDPROC WndProc );
	bool EnableGL();

	// Niszczenie Okna ( nic nie zwraca, bo i tak jak si co zwali, to nic nie zaszkodzi )
	void GLDestroyWindow();
	void DisableGL();

	// Ustawianie perspektywy
	void SetPerspective( float kat, int szerokosc, int wysokosc, float blisko, float daleko );
	void SetPerspective( float kat, float wspl, float blisko, float daleko );
	void SetPerspective();

	// Ustawianie rzut. prostoktnego
	void SetOrtho( float lewo, float prawo, float dol, float gora, float blisko = -1, float daleko = 1 );
	void SetOrtho();

	// Zamiana buforw
	void SwitchBuf();

	// Funkcje by pobra szeroko i wysoko okna
	const int GetWidth();
	const int GetHeight();

	bool GetFullScreen();
	void EnableFullScreen();
	void DisableFullScreen();

	void VSync( unsigned int set );

	std::string GetRndInfo( unsigned int info );
};

extern UIRender GLRender;

#endif