#include "Weapon.h"

/*====================
	KLASA weBullet
	zwyky pocisk - 
	podstawa do 
	reszty pociskw.
====================*/
weBullet::weBullet()
{
	CanDelete = false;
	Visible = true;
	DoDelete = false;
}

void weBullet::Init( Vector3f pos, Vector3f veloc, float speed )
{
	Veloc = veloc * speed;
	Pos = pos + ( veloc * 1.5f );
	NextPos = Pos;
	R = 0.1f;

	Type = BULLET_TYPE_NORMAL;
}

void weBullet::DoEngine()
{
	if( !TestCollBlock( this, GLevel.GetBlock( this->GetBlockPos() ) ) )
	{
		Pos = NextPos;
		NextPos = Pos + ( Veloc * GUI.GetSpeed() );
		if( GLevel.GetBlock( this->GetBlockPos() ) == NULL )
			DoDelete = true;
	}
	else DoDelete = true;

	if( DoDelete )
		OnDelete();
}

float weBullet::DoTest( Dummy* Dum, float Armor )
{
	if( Dum != NULL && Dum != Owner && TestCollDum( Dum, this ) )
	{
		float ArmorMod = 1.0f - ( ( Armor * 0.5f ) / 100.0f );
		DoDelete = true;
		return Damage * ArmorMod;
	}
	return 0.0f;
}

void weBullet::DoDraw()
{
	glPushMatrix();
	GLUquadric* obj = gluNewQuadric();
	glTranslatef( Pos.X, Pos.Y, Pos.Z );

	glDisable( GL_TEXTURE_2D );

	glColor3f( 0.5f, 0.5f, 0.5f );
	gluSphere( obj, 0.1, 8, 8 );

	glEnable( GL_TEXTURE_2D );

	gluDeleteQuadric( obj );
	glPopMatrix();
}

void weBullet::OnDelete()
{
	specSprite* Spr = new specSprite;
	Spr->Create( NextPos, 0.5f, 0.5f, 0.5f );
	SEManager.AddEffect( Spr );
	CanDelete = true;
}


/*====================
	KLASA weBullRay
	promie phazer'a
====================*/
void weBullRay::Init( Vector3f pos, Vector3f veloc, float speed )
{
	Pos = pos + ( veloc * 1.5f );
	Time = 1.0f;
	R = 0.1f;
	Veloc = veloc;
	Type = BULLET_TYPE_RAY;

	specRay* Effect = new specRay;
	specSprite* Spr = new specSprite;

	Effect->Create( Pos, veloc );
	Pos = Effect->FromPos;
	NextPos = Effect->ToPos;

	Spr->Create( Effect->ToPos + ( veloc.Reverse() * 0.2f ), 1.0f, 0.8f, 0.0f );

	SEManager.AddEffect( Effect );
	SEManager.AddEffect( Spr );
}

float weBullRay::DoTest( Dummy* Dum, float Armor )
{
	float ArmorMod = 1.0f - ( ( Armor * 0.5f ) / 100.0f );
	if( Dum != NULL && Dum != Owner )
	{
		if( TestCollDum( Dum, this ) )
		{
			specSprite* Spr = new specSprite;
			Spr->Create( Dum->NextPos, 1.0f, 2.0f, 0.0f );
			SEManager.AddEffect( Spr );
			return Damage * ArmorMod;
		}
	}
	return 0.0f;
}

void weBullRay::DoEngine()
{
	CanDelete = true;
}

void weBullRay::DoDraw()
{
	// Nic...
}


/*====================
	KLASA weBullRocked
	Rakieta z wyrzutni
====================*/
void weBullRocket::Init( Vector3f pos, Vector3f veloc, float speed )
{
	Pos = pos + ( veloc * 3.0f );
	Veloc = veloc * speed;
	NextPos = pos + Veloc;
	Angle = GetAngle( Pos, NextPos );
	Sec = 0.0f;
	R = 0.3f;
	glEnable( GL_LIGHT0 );

	Model = GLMManager.Get( "Data/Missle.glm" );

	Type = BULLET_TYPE_ROCKET;
}

void weBullRocket::DoDraw()
{
	glPushMatrix();
	glTranslatef( Pos.X, Pos.Y, Pos.Z );
	glRotatef( Angle, 0.0f, 1.0f, 0.0f );
	
	glRotatef( 180.f, 0.0f, 1.0f ,0.0f );
	float t[] = { 0.0f, 0.0f, 0.0f, 1.0f };
	glLightfv( GL_LIGHT0, GL_POSITION, t );
	Model->CallObject( 0 );

	glPopMatrix();
}

void weBullRocket::OnDelete()
{
	CanDelete = true;
	weBullExplode *Bull = new weBullExplode;
	Bull->Owner = NULL;
	Bull->Damage = this->Damage / 10.0f;
	Bull->Power = 15.0f;
	Bull->Init( Pos, Veloc, 0.4f );
	BManager.AddBullet( Bull );
	glDisable( GL_LIGHT0 );
}

void weBullRocket::DoEngine()
{
	if( !TestCollBlock( this, GLevel.GetBlock( this->GetBlockPos() ) ) )
	{
		Pos = NextPos;
		NextPos = Pos + ( Veloc * GUI.GetSpeed() );
		Sec += 1.0f * GUI.GetSpeed();
		if( Sec > 3.0f )
		{
			Vector3f Tail = Veloc;
			Tail.Normalize();
			Tail *= 1.0f;
			Tail = Tail.Reverse();
			Sec = 0.0f;
			specSprite* spec = new specSprite;
			spec->Create( Pos + Tail, 1.0f, 0.7f, 0.0f );
			SEManager.AddEffect( spec );
		}
		if( GLevel.GetBlock( this->GetBlockPos() ) == NULL )
			DoDelete = true;
	}
	else DoDelete = true;

	if( DoDelete )
		OnDelete();
}

/*====================
	KLASA weBullExplode
	eksplozja
====================*/
void weBullExplode::Init( Vector3f pos, Vector3f veloc, float speed )
{
	thisPower = 0.0f;
	Step = speed;
	NextPos = Pos = pos;
	glEnable( GL_LIGHT1 );

	specExplode* spec = new specExplode;
	spec->Create( Pos, Power, Step );
	SEManager.AddEffect( spec );
}

float weBullExplode::DoTest( Dummy *Dum, float Armor )
{
	if( Dum == NULL || Dum == Owner )
		return 0.0f;

	float ArmorMod = 1.0f - ( ( Armor * 0.5f ) / 100.0f );
	float dist = mathDist( Pos, Dum->NextPos );

	if( dist < Dum->R + this->thisPower )
	{
		if( dist < Dum->R + ( this->thisPower / 2.0f ) )
			ArmorMod *= 2.0f;

		return Damage * ArmorMod * GUI.GetSpeed();
	}

	return 0.0f;
}

void weBullExplode::DoEngine()
{
	thisPower += Step;

	if( thisPower >= Power * 0.8f )
	{
		CanDelete = true;
		glDisable( GL_LIGHT1 );
	}
}

void weBullExplode::DoDraw()
{
	float t[] = { Pos.X, Pos.Y, Pos.Z, 1.0f };
	glLightfv( GL_LIGHT1, GL_POSITION, t );
	// Nic...
}

/*====================
	KLASA weBullManager
	Zarzdza wszystkimi pociskami
====================*/
void weBulletManager::AddBullet( weBullet *bullet )
{
	List.push_back( bullet );
}

void weBulletManager::DeleteBullet( unsigned int index )
{
	delete List[index];
	List.erase( List.begin() + index );
}

weBullet* weBulletManager::GetBullet( unsigned int index )
{
	return List[index];
}

float weBulletManager::DoTest( Dummy* Dum, float Armor )
{
	float Damage = 0.0f;
	weBullet* bull;
	for( int i = List.size()-1; i >= 0; i-- )
	{
		bull = List[i];

		Damage += bull->DoTest( Dum, Armor );

		if( bull->CanDelete )
			DeleteBullet( i );
	}
	return Damage;
}

void weBulletManager::DoEngine()
{
	weBullet* bull;
	for( int i = List.size()-1; i >= 0; i-- )
	{
		bull = List[i];
		bull->DoEngine();

		if( bull->CanDelete )
			DeleteBullet( i );
	}
}

void weBulletManager::DoDraw()
{
	weBullet* bull;
	for( int i = 0; i < List.size(); i++ )
	{
		bull = List[i];
		if( !bull->Visible )
			continue;
		bull->DoDraw();
	}
	if( GUI.GetReflection() )
	{
		glPushMatrix();
		glTranslatef( 0.0f, -10.0f, 0.0f );
		glScalef( 1.0f, -1.0f, 1.0f );
		glDisable( GL_CLIP_PLANE0 );
		glEnable( GL_CLIP_PLANE1 );
		glFrontFace( GL_CW );
		for( i = 0; i < List.size(); i++ )
		{
			bull = List[i];
			if( !bull->Visible )
				continue;
			bull->DoDraw();
		}
		glFrontFace( GL_CCW );
		glDisable( GL_CLIP_PLANE1 );
		glEnable( GL_CLIP_PLANE0 );
		glPopMatrix();
	}
}

weBulletManager BManager;

/*	KLASA WEAPON
	Jest to klasa do dziedziczenia,
	zawiera wszystko co jest potrzebne
	do broni :)
*/

/*	KONSTRUKTOR	*/
weWeapon::weWeapon()
{
	Damage[0] = 0.0f;
	Damage[1] = 1.0f;
	type = 0;
	Ammo = 0;
	MaxAmmo = 0;
	AmmoPerClip = 0;
	inited = false;
	InHand = false;
	Rot = 0.0f;
	Shake[0] = 0.0f;
	Shake[1] = 0.0f;
	ShotTime = 0.0f;
	ShotPause = 0.0f;
	ReloadTime = 0.0f;
	hand = 0;
	fire = false;
	Model = NULL;
	Owner = NULL;
}

/*	DESTRUKTOR	*/
weWeapon::~weWeapon()
{
	Free();
}

/*	Jako, e bro na ziemi wyglda nudno
	to bdziemy j obraca. Efekt wyglda
	lepiej. :)
*/
void weWeapon::Rotate()
{
	Rot += 0.5f * GUI.GetSpeed();
	if( Rot >= 360.0f )
		Rot = 0.0f;
}

/*	Ta funkcja jest wywoywana, gdy
	jaka posta podniesie bro.
*/
void weWeapon::PickUp( weWeapon* Weapon, gamePlayer* Player )
{
	if( !InHand )
	{
		hand = Player->GetHand();
		InHand = true;
		Owner = Player;
		if( !inited )
			Init();
		GUI.SendMsg( "Podniosles: " + Weapon->Name, 4000, 10.0f, -1.0f, 1.5f, 1.5f, 1.0f, 0.5f, 0.5f );
	}
	else
	{
		if( Weapon->GetAllAmmo() == -1 )
			return;

		int PickAmmo = 0;
		if( this->Ammo + Weapon->GetAllAmmo() <= this->MaxAmmo )
		{
			PickAmmo = Weapon->GetAllAmmo();
			this->Ammo += Weapon->GetAllAmmo();
			Weapon->SetAmmo( 0 );
		}
		else
		{
			int temp = this->MaxAmmo - this->Ammo;
			Ammo += temp;
			PickAmmo = temp;
			Weapon->SetAmmo( Weapon->GetAllAmmo() - temp );
		}
		if( PickAmmo != 0 )
			GUI.SendMsg( "Podniosles " + guiIntToStr( PickAmmo ) + " amunicji.", 4000, 10.0f, -1.0f, 1.5f, 1.5f, 0.5f, 1.0f, 0.5f );		
	}
}

int weWeapon::GetAmmo()
{
	if( Ammo == -1 )
		return -1;
	if( AmmoPerClip != -1 )
		return ( Ammo % AmmoPerClip) + 1;
	else
		return Ammo;
}

int weWeapon::GetAllAmmo()
{
	return Ammo;
}

void weWeapon::SetAmmo( int ammo )
{
	Ammo = ammo;
}

int weWeapon::GetClip()
{
	if( AmmoPerClip == -1 )
		return -1;
	return ( Ammo / AmmoPerClip );
}

bool weWeapon::GetHave()
{
	return InHand;
}

unsigned int weWeapon::GetType()
{
	return type;
}

bool weWeapon::GetInited()
{
	return inited;
}

void weWeapon::DoEngine()
{

}

void weWeapon::DoDraw()
{

}

void weWeapon::Init()
{

}

void weWeapon::Shot()
{

}

void weWeapon::Free()
{
	if( !inited )
		return;

	Model = NULL;
}


/*	PIA :D
	Teraz zaczynaj si klasy broni.
	Kada bro ma odzieln klas, by
	miaa wasny wygld, waciwoci
	i animacje. Na pierwszy ogie idzie
	pia tarczowa ;D
*/

weSaw::weSaw()
{
}

/*	Metoda inicjuje pie, czyli
	Przypisuje wartoci i tworzy jej
	wygld. Mogem zrobi jaki loader
	plikw, ale to by si zeszo zbyt
	dugo :P 
*/
void weSaw::Init()
{
	if( inited ) 
		return;

	type = GAME_WEAP_SAW;
	/*	Najpierw inicjujemy zmienne.
		Te bd odpowiedzialne za
		animacje obracajcej si tarczy
		i przysuwania podczas ataku
	*/
	ShotTime = 0.0f;
	ShotPause = 0.1f;
	ReloadTime = 0.0f;
	/*	Teraz ustawiamy amunicje.
		Jako, e bro nie posiada
		amunicji jako takiej, to
		ustawiamy na -1 (nieskoczone)
	*/
	Ammo = -1;
	MaxAmmo = -1;
	AmmoPerClip = -1;

	Damage[0] = 0.1f;
	Damage[1] = 0.5f;

	Name = "SAW";

	Model = GLMManager.Get( "Data/saw-model.glm" );

	inited = true;
}

/*	METODA ENGINE'A
	By zachowa jak dynamike
	dodaem tu par bajerw: ruszajca
	si bro, przysuwanie przy strzale,
	powolny rozruch tarczy, itp. Lepiej
	to wyglda ni statyczny model i sowo
	"O, trafiem..." ;)
*/
void weSaw::DoEngine()
{
	//	Funkcja obracajca, dziaa bez wzgldu, czy bro jest na ziemi, czy nie
	Rotate();

	// Sprawdzamy, czy bro jest podnisiona
	if( !InHand ) 
		return;

	// Jeeli bro jest w rku, to uruchamiamy dodatkowy kod.

	/*	Gdy strzelamy, bro si przysuwa do przeciwnika,
		a ostrze wiruje coraz szybciej. Przy okazji rka si
		nie trzsie ( jak to dziwnie brzmi :P )
	*/
	if( fire ) 
	{
		if( ShotPause < 20.0f )
			ShotPause += 0.4f;
		if( ReloadTime > -1.0f )
			ReloadTime -= 0.2f;
		Shake[0] = 0.0f;
		Shake[1] = 0.0f;
		fire = false;
	}
	/*	Gdy jednak nic si niedzieje, ostrze zwalnia, a bro
		wraca na miejsce. Przy okazji brn wykonuje delikatne
		"chwianie" si powodujc zudzenie trzymajcej jej rki.
		*/
	else 
	{
		if( ShotPause > 0.3f )
			ShotPause -= 0.1f;
		if( ReloadTime < 0.0f )
			ReloadTime += 0.05f;
		Shake[0] = ( sinf( Rot / 10.0f ) / 20.0f );
		Shake[1] = ( cosf( Rot / 10.0f ) / 20.0f );
	}
	// Obracamy ostrze i gdy wykonao peny obrt, zerujemy
	ShotTime += ShotPause * GUI.GetSpeed();
	if( ShotTime >= 360.0f )
		ShotTime = 0.0f;
}

/*	METODA RYSUJCA
	Tu nastpuje wywietlenie na ekranie. Nic wielkiego :P
*/
void weSaw::DoDraw()
{
	// Zachowujemy aktualn Macierz
	glPushMatrix();
	// Rysujemy bro, w zalenoci, czy ona jest na ziemi, czy nie
	if( !InHand )
	{
		glTranslatef( Pos.X, 0.0f, Pos.Z );

		glTranslatef( 0.0f, -3.0f, 0.0f );
		glRotatef( Rot, 0.0f, 1.0f, 0.0f );
		Model->CallObject( 0 );
	}
	else
	{
		glLoadIdentity();

		switch( hand )
		{
		case GAME_RIGHT_HAND :
			glTranslatef( 1.5f, -1.5f, -5.0f );
			break;
		case GAME_LEFT_HAND :
			glTranslatef( -1.5f, -1.5f, -5.0f );
			break;
		case GAME_CENTER_HAND :
			glTranslatef( 0.0f, -1.5f, -5.0f );
			break;
		default:
			glTranslatef( 1.5f, -1.5f, -5.0f );
			break;
		}

		glTranslatef( Shake[0], Shake[1], ReloadTime * GUI.GetSpeed() );

		Model->CallObject( 1 );		

		glTranslatef( 0.0f, 0.0f, -2.0f );
		glRotatef( ShotTime, 0.0f, 1.0f, 0.0f );

		Model->CallObject( 2 );	
	}

	// Przywracamy Macierz
	glPopMatrix();
}

void weSaw::Shot()
{
	if( !inited )
		return;
	fire = true;
}

/*	PISTOLET	*/
void wePistol::Init()
{
	if( inited ) 
		return;

	type = GAME_WEAP_PISTOL;
	Reloading = false;
	back = false;
	BackA = 0.0f;

	ShotTime = 5.0f;
	ShotPause = 15.0f;
	ReloadTime = 80.0f;

	Ammo = 500;
	AmmoPerClip = 14;
	MaxAmmo = 999;
	CurrClip = Ammo % AmmoPerClip + 1;

	Damage[0] = 2.0f;
	Damage[1] = 3.0f;

	Name = "Pistol";

	Time = 0.0f;

	Model = GLMManager.Get( "Data/pistol-model.glm" );

	inited = true;
}

void wePistol::DoEngine()
{
	Rotate();

	if( !InHand )
		return;

	if( back )
	{
		Time += 1.0f * GUI.GetSpeed();
		BackA += ( 50.0f / ShotPause ) * GUI.GetSpeed();
		if( Time > ShotPause )
		{
			back = false;
			Time = 0.0f;
			BackA = 0.0f;
		}
	}

	if( fire ) 
	{
		if( !back )
		{
			Time += 1.0f * GUI.GetSpeed();
			if( Time >= ShotTime )
			{
				back = true;
				Time = 0.0f;
				Ammo--;
				CurrClip--;
				weBullet* Bull = new weBullet;
				Vector3f temp;
				temp = RayCast( Owner->NextPos, Owner->Vector, 2.0f, &GLevel );
				temp = temp - Pos;
				temp.Normalize();
				Bull->Init( Pos, temp, 4.0f );
				Bull->Owner = this->Owner;
				Bull->Damage = Damage[0];
				BManager.AddBullet( Bull );
			}
		}
		Shake[0] = 0.0f;
		Shake[1] = 0.0f;
		fire = false;
	}
	else if( Reloading && !back )
	{
		Time += 1.0f * GUI.GetSpeed();
		if( Time >= ReloadTime )
		{
			if( Ammo >= AmmoPerClip )
			{
				CurrClip = AmmoPerClip;
				Ammo -= AmmoPerClip;
			}
			else
			{
				CurrClip = Ammo;
			}
			Reloading = false;
		}
	}
	/*	Gdy jednak nic si niedzieje, ostrze zwalnia, a bro
		wraca na miejsce. Przy okazji brn wykonuje delikatne
		"chwianie" si powodujc zudzenie trzymajcej jej rki.
		*/
	else if( !Reloading )
	{
		back = false;
		BackA = 0.0f;
		Shake[0] = ( sinf( Rot / 10.0f ) / 15.0f );
		Shake[1] = ( cosf( Rot / 10.0f ) / 15.0f );
	}
	// Obracamy ostrze i gdy wykonao peny obrt, zerujemy
}

void wePistol::DoDraw()
{
	// Zachowujemy aktualn Macierz
	glPushMatrix();
	// Rysujemy bro, w zalenoci, czy ona jest na ziemi, czy nie
	if( !InHand )
	{
		glTranslatef( Pos.X, 0.0f, Pos.Z );

		glTranslatef( 0.0f, -3.0f, 0.0f );
		glRotatef( Rot, 0.0f, 1.0f, 0.0f );
		Model->CallObject( 0 );
	}
	else
	{
		glLoadIdentity();

		switch( hand )
		{
		case GAME_RIGHT_HAND :
			glTranslatef( 1.5f, -1.5f, -5.0f );
			break;
		case GAME_LEFT_HAND :
			glTranslatef( -1.5f, -1.5f, -5.0f );
			break;
		case GAME_CENTER_HAND :
			glTranslatef( 0.0f, -1.5f, -5.0f );
			break;
		default:
			glTranslatef( 1.5f, -1.5f, -5.0f );
			break;
		}

		if( back )
			glRotatef( BackA, 1.0f, 0.0f, 0.0f );

		if( Reloading && !back)
		{
			glTranslatef( 0.0f, -Time / 50.0f, 0.0f );
			glRotatef( Time, 0.0f, 1.0f, 0.0f );
		}

		glTranslatef( Shake[0], Shake[1], 0.0f );

		Model->CallObject( 0 );		
	}

	// Przywracamy Macierz
	glPopMatrix();
}

void wePistol::Shot()
{
	if( Reloading )
		return;

	if( CurrClip == 0 && Ammo > 0 )
		Reloading = true;

	if( Ammo > 0 || Ammo == -1 )
		fire = true;
}



void weMiniPhazer::Init()
{
	if( inited ) 
		return;

	type = GAME_WEAP_MINIPZR;
	back = false;
	BackA = 0.0f;

	ShotTime = 5.0f;
	ShotPause = 60.0f;
	ReloadTime = 0.0f;

	Ammo = 999;
	AmmoPerClip = -1;
	MaxAmmo = 999;
	CurrClip = -1;

	Damage[0] = 40.0f;
	Damage[1] = 60.0f;

	Name = "MiniPhazer";

	Time = 0.0f;

	Model = GLMManager.Get( "Data/miniphazer-model.glm" );

	inited = true;
}

void weMiniPhazer::DoEngine()
{
	Rotate();

	if( InHand )
	{
		Shake[0] = ( sin( Rot / 20.0f ) / 10.0f );
		Shake[1] = ( cos( Rot / 10.0f ) / 20.0f );
		if( fire )
		{
			if( !back )
			{
				Shake[0] = 0.0f;
				Shake[1] = 0.0f;
				Vector3f temp;
				temp = RayCast( Owner->Pos, Owner->Vector, 0.5f, &GLevel );
				temp = temp - Pos;
				temp.Normalize();
				weBullet* Bull = new weBullRay;
				Bull->Init( Pos, temp, 0 );
				Bull->Owner = this->Owner;
				Bull->Damage = this->Damage[0] + float( rand() % int( this->Damage[1] - this->Damage[0]) );
				BManager.AddBullet( Bull );
				back = true;
				Ammo--;
			}
			else
			{
				Time += 1.0f * GUI.GetSpeed();
				if( Time >= ShotPause * 0.2f )
				{
					BackA -= ( 1.0f / ( ShotPause * 0.8f )) * GUI.GetSpeed();
					if( Time >= ShotPause )
					{
						BackA = 0.0f;
						Time = 0.0f;
						fire = false;
						back = false;
					}
				}
				else
				{
					BackA += ( 1.0f / ( ShotPause * 0.2f ) ) * GUI.GetSpeed();
				}
			}
		}

	}
}

void weMiniPhazer::DoDraw()
{
	// Zachowujemy aktualn Macierz
	glPushMatrix();
	// Rysujemy bro, w zalenoci, czy ona jest na ziemi, czy nie
	if( !InHand )
	{
		glTranslatef( Pos.X, 0.0f, Pos.Z );

		glTranslatef( 0.0f, -3.0f, 0.0f );
		glRotatef( Rot, 0.0f, 1.0f, 0.0f );
		Model->CallObject( 0 );
	}
	else
	{
		glLoadIdentity();

		switch( hand )
		{
		case GAME_RIGHT_HAND :
			glTranslatef( 1.5f, -1.5f, -4.5f );
			break;
		case GAME_LEFT_HAND :
			glTranslatef( -1.5f, -1.5f, -4.5f );
			break;
		case GAME_CENTER_HAND :
			glTranslatef( 0.0f, -1.5f, -4.5f );
			break;
		default:
			glTranslatef( 1.5f, -1.5f, -4.5f );
			break;
		}

		if( back )
			glTranslatef( 0.0f, 0.0f, BackA );

		glTranslatef( Shake[0], Shake[1], 0.0f );

		Model->CallObject( 0 );
	}

	// Przywracamy Macierz
	glPopMatrix();
}

void weMiniPhazer::Shot()
{
	if( Ammo == 0 )
		return;
	if( !back )
		fire = true;
}




void wePhazer::Init()
{
	if( inited ) 
		return;

	type = GAME_WEAP_PHAZER;
	back = false;
	BackA = 0.0f;

	ShotTime = 5.0f;
	ShotPause = 60.0f;
	ReloadTime = 0.0f;

	Ammo = 999;
	AmmoPerClip = -1;
	MaxAmmo = 999;
	CurrClip = -1;

	Damage[0] = 80.0f;
	Damage[1] = 120.0f;

	Name = "Phazer";

	Time = 0.0f;

	Model = GLMManager.Get( "Data/phazer-model.glm" );

	inited = true;
}

void wePhazer::DoEngine()
{
	Rotate();

	if( InHand )
	{
		Shake[0] = ( sin( Rot / 20.0f ) / 10.0f );
		Shake[1] = ( cos( Rot / 10.0f ) / 20.0f );
		if( fire )
		{
			if( !back )
			{
				Shake[0] = 0.0f;
				Shake[1] = 0.0f;
				Vector3f temp;
				temp = RayCast( Owner->Pos, Owner->Vector, 0.5f, &GLevel );
				temp = temp - Pos;
				temp.Normalize();
				weBullet* Bull = new weBullRay;
				Bull->Init( Pos, temp, 0 );
				Bull->Owner = this->Owner;
				Bull->Damage = this->Damage[0] + float( rand() % int( this->Damage[1] - this->Damage[0]) );
				BManager.AddBullet( Bull );
				back = true;
				Ammo--;
			}
			else
			{
				Time += 1.0f * GUI.GetSpeed();
				if( Time >= ShotPause * 0.2f )
				{
					BackA -= ( 2.0f / ( ShotPause * 0.8f )) * GUI.GetSpeed();
					if( Time >= ShotPause )
					{
						BackA = 0.0f;
						Time = 0.0f;
						fire = false;
						back = false;
					}
				}
				else
				{
					BackA += ( 2.0f / ( ShotPause * 0.2f ) ) * GUI.GetSpeed();
				}
			}
		}

	}
}

void wePhazer::DoDraw()
{
	// Zachowujemy aktualn Macierz
	glPushMatrix();
	// Rysujemy bro, w zalenoci, czy ona jest na ziemi, czy nie
	if( !InHand )
	{
		glTranslatef( Pos.X, 0.0f, Pos.Z );

		glTranslatef( 0.0f, -3.0f, 0.0f );
		glRotatef( Rot, 0.0f, 1.0f, 0.0f );
		Model->CallObject( 0 );
	}
	else
	{
		glLoadIdentity();

		switch( hand )
		{
		case GAME_RIGHT_HAND :
			glTranslatef( 1.5f, -1.5f, -3.5f );
			break;
		case GAME_LEFT_HAND :
			glTranslatef( -1.5f, -1.5f, -3.5f );
			break;
		case GAME_CENTER_HAND :
			glTranslatef( 0.0f, -1.5f, -3.5f );
			break;
		default:
			glTranslatef( 1.5f, -1.5f, -3.5f );
			break;
		}

		if( back )
			glTranslatef( 0.0f, 0.0f, BackA );

		glTranslatef( Shake[0], Shake[1], 0.0f );

		Model->CallObject( 0 );
	}

	// Przywracamy Macierz
	glPopMatrix();
}

void wePhazer::Shot()
{
	if( Ammo == 0 )
		return;
	if( !back )
		fire = true;
}

void weMiniGun::Init()
{
	if( inited ) 
		return;

	type = GAME_WEAP_MINIGUN;
	Rot2 = 0.0f;

	ShotTime = 0.0001f;
	ShotPause = 1.0f;
	ReloadTime = 0.0f;

	Ammo = 6000;
	AmmoPerClip = -1;
	MaxAmmo = 6000;
	CurrClip = -1;

	Damage[0] = 0.5f;
	Damage[1] = 1.0f;

	Name = "MiniGun";

	back = false;
	BackA = 0.0f;
	Time = 0.0f;

	Model = GLMManager.Get( "Data/minigun-model.glm" );

	inited = true;
}

void weMiniGun::DoEngine()
{
	Rotate();

	if( InHand )
	{
		if( fire )
		{
			if( Rot2 < 15.0f )
			{
				Rot2 += 1.0f * GUI.GetSpeed();
			}
			else Rot2 = 15.0f;
			if( BackA == 1.0f && Rot2 == 15.0f )
			{
				Time += 1.0f * GUI.GetSpeed();
			}
			if( Time >= ShotPause )
			{
				Time = 0.0f;
				Shake[0] = 0.0f;
				Shake[1] = 0.0f;
				Vector3f temp;
				temp = RayCast( Owner->Pos, Owner->Vector, 6.0f, &GLevel );
				temp = temp - Pos;
				temp.Normalize();
				weBullet* Bull = new weBullet; 
				Bull->Visible = false;;
				Bull->Init( Pos, temp, 4.0f );
				Bull->Owner = this->Owner;
				Bull->Damage = this->Damage[0] * 10.0f;// + float( rand() % int( this->Damage[1] - this->Damage[0]) );
				BManager.AddBullet( Bull );
				Ammo -= 10;
			}
			if( BackA < 1.0f )
			{
				BackA += ( 0.5f / 14.0f ) * GUI.GetSpeed();
				back = true;
			}
			else BackA = 1.0f;
			fire = false;
		}
		else
		{
			Shake[0] = ( sin( Rot / 20.0f ) / 10.0f );
			Shake[1] = ( cos( Rot / 10.0f ) / 20.0f );
			if( Rot2 > 0.0f )
			{
				Rot2 -= 1.0f * GUI.GetSpeed();
			}
			else Rot2 = 0.0f;

			if( BackA > 0.0f )
			{
				BackA -= ( 0.5f / 14.0f ) * GUI.GetSpeed();
				back = false;
			}
			else BackA = 0.0f;

		}
	}
}

void weMiniGun::DoDraw()
{
	glPushMatrix();
	// Rysujemy bro, w zalenoci, czy ona jest na ziemi, czy nie
	if( !InHand )
	{
		glTranslatef( Pos.X, 0.0f, Pos.Z );

		glTranslatef( 0.0f, -3.0f, 0.0f );
		glRotatef( Rot, 0.0f, 1.0f, 0.0f );
		Model->CallObject( 0 );
	}
	else
	{
		glLoadIdentity();

		switch( hand )
		{
		case GAME_RIGHT_HAND :
			glTranslatef( 1.5f, -1.5f, -3.5f );
			break;
		case GAME_LEFT_HAND :
			glTranslatef( -1.5f, -1.5f, -3.5f );
			break;
		case GAME_CENTER_HAND :
			glTranslatef( 0.0f, -1.5f, -3.5f );
			break;
		default:
			glTranslatef( 1.5f, -1.5f, -3.5f );
			break;
		}

		glTranslatef( Shake[0], Shake[1], BackA );

		Model->CallObject( 1 );
		glRotatef( Rot * Rot2, 0.0f, 0.0f, 1.0f );
		Model->CallObject( 2 );
	}

	glPopMatrix();
}

void weMiniGun::Shot()
{
	if( Ammo < 0 && Ammo != -1 )
		Ammo = 0;

	if( Ammo == 0 )
		return;

	fire = true;
}



void weRocketLuncher::Init()
{
	if( inited ) 
		return;

	type = GAME_WEAP_ROCKETLUN;
	back = false;
	BackA = 0.0f;

	ShotTime = 8.0f;
	ShotPause = 120.0f;
	ReloadTime = 0.0f;

	Ammo = 999;
	AmmoPerClip = -1;
	MaxAmmo = 999;
	CurrClip = -1;

	Damage[0] = 5.0f;
	Damage[1] = 15.0f;

	Name = "Wyrzutnia Rakiet";

	Time = 0.0f;

	Model = GLMManager.Get( "Data/rocketlun-model.glm" );

	inited = true;
}

void weRocketLuncher::DoEngine()
{
	Rotate();

	if( InHand )
	{
		Shake[0] = ( sin( Rot / 20.0f ) / 10.0f );
		Shake[1] = ( cos( Rot / 10.0f ) / 20.0f );
		if( fire )
		{
			if( !back )
			{
				Shake[0] = 0.0f;
				Shake[1] = 0.0f;
				Vector3f temp;
				temp = RayCast( Owner->Pos, Owner->Vector, 0.5f, &GLevel );
				temp = temp - Pos;
				temp.Normalize();
				weBullet* Bull = new weBullRocket;
				Bull->Init( Pos, temp, 0.5f );
				Bull->Owner = this->Owner;
				Bull->Damage = this->Damage[0] + float( rand() % int( this->Damage[1] - this->Damage[0]) );
				BManager.AddBullet( Bull );
				back = true;
				Ammo--;
			}
			else
			{
				Time += 1.0f * GUI.GetSpeed();
				if( Time >= ShotPause * 0.2f )
				{
					BackA -= ( 2.0f / ( ShotPause * 0.8f )) * GUI.GetSpeed();
					if( Time >= ShotPause )
					{
						BackA = 0.0f;
						Time = 0.0f;
						fire = false;
						back = false;
					}
				}
				else
				{
					BackA += ( 2.0f / ( ShotPause * 0.2f ) ) * GUI.GetSpeed();
				}
			}
		}

	}
}

void weRocketLuncher::DoDraw()
{
	// Zachowujemy aktualn Macierz
	glPushMatrix();
	// Rysujemy bro, w zalenoci, czy ona jest na ziemi, czy nie
	if( !InHand )
	{
		glTranslatef( Pos.X, 0.0f, Pos.Z );

		glTranslatef( 0.0f, -3.0f, 0.0f );
		glRotatef( Rot, 0.0f, 1.0f, 0.0f );
		Model->CallObject( 0 );
	}
	else
	{
		glLoadIdentity();

		switch( hand )
		{
		case GAME_RIGHT_HAND :
			glTranslatef( 1.5f, -1.5f, -3.5f );
			break;
		case GAME_LEFT_HAND :
			glTranslatef( -1.5f, -1.5f, -3.5f );
			break;
		case GAME_CENTER_HAND :
			glTranslatef( 0.0f, -1.5f, -3.5f );
			break;
		default:
			glTranslatef( 1.5f, -1.5f, -3.5f );
			break;
		}

		if( back )
			glTranslatef( 0.0f, 0.0f, BackA );

		glTranslatef( Shake[0], Shake[1], 0.0f );

		glRotatef( 180.0f, 0.0f, 1.0f, 0.0f );
		Model->CallObject( 0 );
	}

	// Przywracamy Macierz
	glPopMatrix();
}

void weRocketLuncher::Shot()
{
	if( Ammo == 0 )
		return;
	if( !back )
		fire = true;
}



void weAtomBomb::Init()
{
	if( inited )
		return;

	type = GAME_WEAP_ATOM_BOMB;

	ShotTime = 5.0f;
	ShotPause = 10.0f;
	ReloadTime = 0.0f;

	Ammo = 1;
	AmmoPerClip = -1;
	MaxAmmo = 1;
	CurrClip = -1;

	puted = false;
	armed = false;

	Time = 0.0f;

	Name = "NUKE";

	Model = GLMManager.Get( "Data/atombomb-model.glm" );

	inited = true;
}

void weAtomBomb::DoEngine()
{
	Rotate();

	if( InHand )
	{
		Shake[0] = sin( Rot / 20.0f ) / 10.0f;
		Shake[1] = cos( Rot / 10.0f ) / 20.0f;
		if( fire )
		{
			if( !puted )
			{
				Shake[0] = 0.0f;
				Shake[1] = 0.0f;
				PutPos = Owner->Pos;
				puted = true;
				armed = false;
				Ammo--;
			}
			else
			{
				static int LastSec = GUI.GetTime( GAME_SECOND );
				if( LastSec != GUI.GetTime( GAME_SECOND ) )
				{
					Time += 1.0f;
					LastSec = GUI.GetTime( GAME_SECOND );
				}

				GUI.PInfo.HEALTH = 15.0f - Time;
				if( !armed && Time >= ShotTime )
				{
					armed = true;
				}
				if( armed )
				{
					if( Time >= ShotTime + ShotPause )
					{
						specExplode* Exp = new specExplode;
						Exp->Create( PutPos, 100.0f, 0.1f );
						SEManager.AddEffect( Exp );
						armed = false;
						puted = false;
						fire = false;
						Time = 0.0f;
					}
				}
			}
		}
	}
}

void weAtomBomb::DoDraw()
{
	// Zachowujemy aktualn Macierz
	glPushMatrix();
	// Rysujemy bro, w zalenoci, czy ona jest na ziemi, czy nie
	if( !InHand )
	{
		glTranslatef( Pos.X, 0.0f, Pos.Z );

		glTranslatef( 0.0f, -3.0f, 0.0f );
		glRotatef( Rot, 0.0f, 1.0f, 0.0f );
		Model->CallObject( 0 );
	}
	else if( !puted )
	{
		glLoadIdentity();

		switch( hand )
		{
		case GAME_RIGHT_HAND :
			glTranslatef( 1.5f, -1.5f, -3.5f );
			break;
		case GAME_LEFT_HAND :
			glTranslatef( -1.5f, -1.5f, -3.5f );
			break;
		case GAME_CENTER_HAND :
			glTranslatef( 0.0f, -1.5f, -3.5f );
			break;
		default:
			glTranslatef( 1.5f, -1.5f, -3.5f );
			break;
		}

		glTranslatef( Shake[0], Shake[1], 0.0f );

		Model->CallObject( 0 );
	}

	// Przywracamy Macierz
	glPopMatrix();
}

void weAtomBomb::Shot()
{
	fire = true;
}

/*
void gameWeapon::BuildFazer()
{
	list[0] = glGenLists( 1 );
	list[1] = glGenLists( 1 );

	GLUquadric* obj = gluNewQuadric();
	gluQuadricTexture( obj, GL_TRUE );


	glNewList( list[0], GL_COMPILE );

	glPushMatrix();

	Tex[0].Activate();

	gluCylinder( obj, 0.4, 0.4, 2, 32, 32 );

	gluQuadricOrientation( obj, GLU_INSIDE );

	gluCylinder( obj, 0.2, 0.2, 2, 32, 32 );

	gluQuadricOrientation( obj, GLU_OUTSIDE );

	glTranslatef( 0, 0, 2 );

	gluDisk( obj, 0.2, 0.4, 32, 2 );

	glTranslatef( 0, 0, -2.5 );

	Tex[1].Activate();

	gluSphere( obj, 0.8, 32, 32 );

	gluPartialDisk( obj, 0.8, 1, 32, 2, 0, 90 );
	gluQuadricOrientation( obj, GLU_INSIDE );
	gluPartialDisk( obj, 0.8, 1, 32, 2, 0, 90 );
	gluQuadricOrientation( obj, GLU_OUTSIDE );

	glRotatef( -90, 1, 0, 0 );

	gluDisk( obj, 0.8, 1, 32, 2 );

	glPopMatrix();

	glEndList();

	glNewList( list[1], GL_COMPILE );

	glPushMatrix();

	Tex[0].Activate();

	gluCylinder( obj, 0.5, 0.5, 2, 32, 32 );

	gluQuadricOrientation( obj, GLU_INSIDE );

	gluCylinder( obj, 0.2, 0.2, 2, 32, 32 );

	gluQuadricOrientation( obj, GLU_OUTSIDE );

	glTranslatef( 0, 0, 2 );

	gluDisk( obj, 0.2, 0.5, 32, 2 );

	glTranslatef( 0, 0, -2.5 );

	Tex[1].Activate();

	gluSphere( obj, 0.8, 32, 32 );

	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
	gluPartialDisk( obj, 0.8, 0.9, 32, 2, 0, 90 );
	gluPartialDisk( obj, 1.9, 2.0, 32, 2, 0, 90 );
	gluQuadricTexture( obj, GL_FALSE );
	glEnable( GL_BLEND );
	glColor4f( 0.8f, 0.8f, 1.0f, 0.5f );
	gluPartialDisk( obj, 0.9, 1.9, 32, 2, 0, 90 );
	glColor4f( 1, 1, 1, 1 );
	glDisable( GL_BLEND );
	gluQuadricTexture( obj, GL_TRUE );
	gluQuadricOrientation( obj, GLU_INSIDE );
	gluPartialDisk( obj, 0.8, 0.9, 32, 2, 0, 90 );
	gluPartialDisk( obj, 1.9, 2.0, 32, 2, 0, 90 );
	gluQuadricTexture( obj, GL_FALSE );
	glEnable( GL_BLEND );
	glColor4f( 0.8f, 0.8f, 1.0f, 0.5f );
	gluPartialDisk( obj, 0.9, 1.9, 32, 2, 0, 90 );
	glColor4f( 1, 1, 1, 1 );
	glDisable( GL_BLEND );
	gluQuadricTexture( obj, GL_TRUE );
	gluQuadricOrientation( obj, GLU_OUTSIDE );

	glRotatef( -90, 1, 0, 0 );

	gluDisk( obj, 0.8, 1, 32, 2 );

	glPopMatrix();

	glEndList();

	gluDeleteQuadric( obj );
}

void gameWeapon::BuildPistol()
{
	list[0] = glGenLists( 1 );
	list[1] = glGenLists( 1 );

	GLUquadric* obj = gluNewQuadric();
	gluQuadricTexture( obj, GL_TRUE );

	Tex[0].LoadTGA( "Data/metal8c.tga" );
	Tex[1].LoadTGA( "Data/metal1c.tga" );

	glNewList( list[0], GL_COMPILE );

	glPushMatrix();

	//=============LUFY==============
	Tex[0].Activate();

	glPushMatrix();

	glTranslatef( -0.2f, 0.0f, 0.0f );

	gluCylinder( obj, 0.2f, 0.2f, 1.0f, 16.0f, 16.0f );

	gluQuadricOrientation( obj, GLU_INSIDE );

	gluCylinder( obj, 0.1f, 0.1f, 1.0f, 16.0f, 16.0f );

	gluQuadricOrientation( obj, GLU_OUTSIDE );

	glTranslatef( 0.0f, 0.0f, 1.0f );

	gluDisk( obj, 0.1f, 0.2f, 16.0f, 2.0f );

	glTranslatef( 0, 0, -1.0f );

	glTranslatef( 0.4, 0.0f, 0.0f );

	gluCylinder( obj, 0.2f, 0.2f, 1.0f, 16.0f, 16.0f );

	gluQuadricOrientation( obj, GLU_INSIDE );

	gluCylinder( obj, 0.1f, 0.1f, 1.0f, 16.0f, 16.0f );

	gluQuadricOrientation( obj, GLU_OUTSIDE );

	glTranslatef( 0.0f, 0.0f, 1.0f );

	gluDisk( obj, 0.1f, 0.2f, 16.0f, 2.0f );

	glPopMatrix();

	//==========KONIEC LUFY=============

	Tex[1].Activate();

	//==========KOLBA============

	gluSphere( obj, 0.4, 16, 16 );

	glTranslatef( 0.0f, 0.0f, -0.3f );

	gluSphere( obj, 0.4, 16, 16 );

	glRotatef( 120, 1.0f, 0.0f, 0.0f );

	gluCylinder( obj, 0.2, 0.2, 0.8, 16, 16 );

	glTranslatef( 0.0f, 0.0f, 0.8f );

	gluDisk( obj, 0.0f, 0.2, 16, 2 );

	//=======KONIEC KOLBA=========

	glPopMatrix();

	glEndList();

	glNewList( list[1], GL_COMPILE );

	glPushMatrix();

	//=============LUFY==============
	Tex[0].Activate();

	glPushMatrix();

	glTranslatef( -0.2f, 0.0f, 0.0f );

	gluCylinder( obj, 0.2f, 0.2f, 1.0f, 16.0f, 16.0f );

	gluQuadricOrientation( obj, GLU_INSIDE );

	gluCylinder( obj, 0.1f, 0.1f, 1.0f, 16.0f, 16.0f );

	gluQuadricOrientation( obj, GLU_OUTSIDE );

	glTranslatef( 0.0f, 0.0f, 1.0f );

	gluDisk( obj, 0.1f, 0.2f, 16.0f, 2.0f );

	glTranslatef( 0, 0, -1.0f );

	glTranslatef( 0.4, 0.0f, 0.0f );

	gluCylinder( obj, 0.2f, 0.2f, 1.0f, 16.0f, 16.0f );

	gluQuadricOrientation( obj, GLU_INSIDE );

	gluCylinder( obj, 0.1f, 0.1f, 1.0f, 16.0f, 16.0f );

	gluQuadricOrientation( obj, GLU_OUTSIDE );

	glTranslatef( 0.0f, 0.0f, 1.0f );

	gluDisk( obj, 0.1f, 0.2f, 16.0f, 2.0f );

	glPopMatrix();

	//==========KONIEC LUFY=============

	Tex[1].Activate();

	//==========KOLBA============

	gluSphere( obj, 0.4, 16, 16 );

	glTranslatef( 0.0f, 0.0f, -0.3f );

	gluSphere( obj, 0.4, 16, 16 );

	glRotatef( 120, 1.0f, 0.0f, 0.0f );

	gluCylinder( obj, 0.2, 0.2, 0.8, 16, 16 );

	glTranslatef( 0.0f, 0.0f, 0.8f );

	gluDisk( obj, 0.0f, 0.2, 16, 2 );

	//=======KONIEC KOLBA=========

	glPopMatrix();

	glEndList();

	gluDeleteQuadric( obj );
}

void gameWeapon::DrawOnGround()
{
	static float rot = 0.0f;
	rot += 0.1f;
	glPushMatrix();

	glTranslatef( 0, -3, 0 );

	glRotatef( rot, 0, 1, 0 );

	glCallList( list[0] );

	glPopMatrix();
}

void gameWeapon::DrawInHand( unsigned int hand )
{
	static float lataj = 0.0f;
	static float lataj2 = 0.0f;
	lataj += 0.002;
	lataj2 += 0.002;

	glPushMatrix();

	switch( hand )
	{
	case GAME_RIGHT_HAND :
		glTranslatef( 1.5, -1.5, -4 );
		glRotatef( 180.0f, 0.0f, 1.0f, 0.0f );
		break;
	case GAME_LEFT_HAND :
		glTranslatef( -1.5, -1.5, -4 );
		glRotatef( 180.0f, 1.0f, 0.0f, 0.0f );
		break;
	case GAME_CENTER_HAND :
		glTranslatef( 0, -1.5, -4 );
		glRotatef( 180.0f, 0.0f, 1.0f, 0.0f );
		break;
	}

	glTranslatef( sinf( lataj ) / 10.0f, cosf( lataj2 ) / 100.0f, 0 );
	glCallList( list[1] );

	glPopMatrix();
}
*/