#ifndef _WEAPON_H_
#define _WEAPON_H_

#include	"defines.h"
#include	"gui.h"
#include	"Render.h"
#include	"Texture.h"
#include	"glm.h"
#include	"3dMath.h"
#include	"Special.h"
#include	"Game.h"

class weBullet : public Dummy
{
protected:
	unsigned int Type;
	Vector3f Veloc;
	bool DoDelete;
public:
	float Damage;
	gameThing* Owner;

	weBullet();

	virtual void Init( Vector3f pos, Vector3f veloc, float speed );

	virtual void DoEngine();
	virtual float DoTest( Dummy* Dum, float Armor = 0.0f );
	virtual void DoDraw();

	virtual void OnDelete();

	bool CanDelete;
	bool Visible;
};

class weBullRay : public weBullet
{
private:
	float Time;
public:
	void Init( Vector3f pos, Vector3f veloc, float speed );
	
	void DoEngine();
	float DoTest( Dummy* Dum, float Armor = 0.0f );
	void DoDraw();
};

class weBullRocket : public weBullet
{
private:
	GLModel* Model;
	float Angle;
	float Sec;
public:
	void Init( Vector3f pos, Vector3f veloc, float speed );

	void DoEngine();
	void DoDraw();
	void OnDelete();
};

class weBullExplode : public weBullet
{
private:
	float thisPower;
	float Step;
public:
	float Power;

	void Init( Vector3f pos, Vector3f veloc, float speed );

	void DoEngine();
	float DoTest( Dummy* Dum, float Armor = 0.0f );
	void DoDraw();
};

class weBulletManager
{
private:
	std::vector<weBullet*> List;
public:
	void AddBullet( weBullet* bullet );
	void DeleteBullet( unsigned int index );
	weBullet* GetBullet( unsigned int index );

	float DoTest( Dummy* Dum, float Armor = 0.0f );
	void DoEngine();
	void DoDraw();
};

extern weBulletManager BManager;

/*	KLASA BRONI
	To jest tylko klasa do 
	dziedziczenia i zawiera
	elementy wsplne dla
	wszystkich klas.
*/
class weWeapon : public Dummy
{
	/*	eby by dostp do tych zmiennych
		w klasach dziedziczczych, umieciem
		je w grupie protected.
	*/
protected:	
	// To jest stopie obrotu, kiedy bro ley na ziemi
	float Rot;
	// To jest stopie drenia w rkach broni
	float Shake[2];
	// Typ Broni ( patrz -> definicje staych )
	unsigned int type;

	/*	Okrela ile zadaje bro obrae.
		Damage[0] - minimum
		Damage[1] - maximum
	*/
	float Damage[2];

	// Lista wywietlania, gdy bro ley na ziemi.
	GLModel	*Model;

	gamePlayer* Owner;

	//	Amunicja oglna ( -1 dla nieskoczonej liczby naboi )
	int Ammo;
	//	Maksymalna amunicja
	int MaxAmmo;
	//	Amunicja na jeden magazynek ( -1, gdy bro nie ma magazynkw )
	int AmmoPerClip;
	//	Amunicja w aktualnym magazynku
	int CurrClip;

	// Dugo jednego strzau
	float ShotTime;
	// Pauza midzy strzaami
	float ShotPause;
	// Czas potrzebny na przeadowanie magazynku
	float ReloadTime;

	// Czy bro zostaa zainicjowana
	bool inited;
	// Czy bro jest w rkach?
	bool InHand;
	// W Ktrej rce trzyma bro?
	unsigned int hand;
	// Czy bro aktualnie strzela
	bool fire;
public:
	std::string Name;
	// Konstruktor i destruktor
	weWeapon();
	~weWeapon();

	// Funkcja podnoszenia broni
	void PickUp( weWeapon* Weapon, gamePlayer* Player );

	virtual void Init();
	virtual void DoEngine();
	virtual void DoDraw();
	virtual void Shot();
	virtual void Free();

	// Funkcja obracajca bro ( animacja raczej wsplna dla wszystkich broni );
	void Rotate();

	int GetAmmo();
	int GetAllAmmo();
	void SetAmmo( int ammo );
	int GetClip();
	bool GetHave();
	unsigned int GetType();
	bool GetInited();
};

/*	KLASA BRONI -> KLASA PIY	*/
class weSaw : public weWeapon
{
private:
public:
	weSaw();
	// Funckja inicjujca
	void Init();
	void DoEngine();
	void DoDraw();

	// Funkcja wywoywana podczas strzau
	void Shot();
};

/*	KLASA BRONI -> KLASA PISTOLETU	*/
class wePistol : public weWeapon
{
private:
	float	Time;
	bool	Reloading;
	bool	back;
	float	BackA;
public:
	// Inicjalizacja
	void Init();
	void DoEngine();
	void DoDraw();

	void Shot();
};

/*	KLASA BRONI -> KLASA PHAZER'A	*/
class weMiniPhazer : public weWeapon
{
private:
	// Tekstury
	float Time;
	bool back;
	float BackA;
public:
	// Inicjalizacja
	void Init();
	void DoEngine();
	void DoDraw();
	void Shot();
};

/*	KLASA BRONI -> KLASA PHAZER'A	*/
class wePhazer : public weWeapon
{
private:
	// Tekstury
	float Time;
	bool back;
	float BackA;
public:
	// Inicjalizacja
	void Init();
	void DoEngine();
	void DoDraw();
	void Shot();
};

/*	KLASA BRONI -> KLASA MINIGUN'A	*/
class weMiniGun : public weWeapon
{
private:
	float Rot2;
	bool back;
	float BackA;
	float Time;
public:
	void Init();
	void DoEngine();
	void DoDraw();
	void Shot();
};

class weRocketLuncher : public weWeapon
{
private:
	float Time;
	bool back;
	float BackA;
public:
	void Init();
	void DoEngine();
	void DoDraw();
	void Shot();
};

/*	KLASA BRONI -> KLASA BOMBY ATOMOWEJ	*/
class weAtomBomb : public weWeapon
{
private:
	float Time;
	bool armed;
	bool puted;
	Vector3f PutPos;
public:
	void Init();
	void DoEngine();
	void DoDraw();
	void Shot();
};

#endif