#ifndef _GLM_H_
#define _GLM_H_

#include "gui.h"		// Nagwek z dostpem do klast Logger
#include "Texture.h"	// Naglwek z klasa ioTexture

// Numer wersji loader'a
#define		GLM_VERSION		100

/*	Gwna klasa modeli OpenGL
*/
class GLModel
{
protected:
	// Lista tekstur
	ioTexture** Tex;
	// Liczba tekstur
	unsigned int TexCount;
	// Lista obiektw
	unsigned int List;
	// Liczba obiektw
	unsigned int ListCount;
	// Lista klatek
	unsigned int Frame;
	// Liczba klatek
	unsigned int FrameCount;
	// Aktualna klatka
	unsigned int CurrFrame;
	// Animacja od klatki
	unsigned int FromFrame;
	// Animacja do Klatki
	unsigned int ToFrame;
	// Czy model jest zaadowany?
	bool loaded;
	// Czy ma animacje?
	bool Animation;
	// Czy aktualnie trwa animacja
	bool playing;
	// Ostatni zaadowany plik
	std::string file;
	// Obiekt do rysowania kwadryk ( tymczasowy, ale jest tu, by by dostpny w caej klasie )
	GLUquadric* obj;

	std::string GetString( FILE* fp );
	std::string NoSpace( std::string str );
	void ParseGLCommand( std::string str );
	bool GetParams( std::string str, int from, std::string* param, int paramCount, std::string Com );
	int GetConst( std::string str, std::string Com );

	bool ReadHeader( FILE* fp );
	bool ReadAnimHeader( FILE* fp );
public:
	GLModel();
	~GLModel();

	bool LoadModel( std::string filename );
	virtual void Free();
	std::string GetFile();
	unsigned int GetObjCount();

	void CallObject( unsigned int index );
	void DoDrawAnim( unsigned int index );
	void DoEngineAnim();
	void PlayAnim( unsigned int fromframe, unsigned int toframe, bool canskip = false );
};


class GLModelManager
{
private:
	std::vector<GLModel*> List;
public:
	GLModelManager();
	~GLModelManager();
	GLModel* Get( std::string filename );
	void AddModel( GLModel* Model );
	GLModel* GetModel( unsigned int index );
	void DeleteModel( unsigned int index );
	void Clear();
};

extern GLModelManager GLMManager;

#endif